/**
 * HTMLユーティリティ
 * HTMLを操作する上で、便利なユーティリティ。
 * ある程度のWebブラウザ間の違いは、この中で吸収します。
 *
 * Version 0.9.0.0
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2012 Yagyu Teru, http://yscjp.com/
 *
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.html) com.yscjp.jsapp.html = {};

/**
 * テーブルエレメントを、DOMで使いやすくするラッパクラス
 * コンストラクタ
 * @param table : 要素 : テーブルエレメント
 */
com.yscjp.jsapp.html.Table = function (table) {

    if (table) {

        if (table.tagName) {

            if (table.tagName.toLowerCase() == "table") {

                this.table = table;
                return;
            }
        }
    }

    throw "com.yscjp.jsapp.html.Table:Illegal argment table is null";
};
(function() {

    var Table = com.yscjp.jsapp.html.Table;

    /**
     * テーブルヘッダを作成し返します。
     * 既に存在する場合は、それを返します。
     */
    Table.prototype.createHead = function() {

        var thead = this.table.tHead;
        if (null == thead) {

            thead = this.table.createTHead();
        }
        return thead;
    };

    /**
     * テーブルフッタを作成し返します。
     * 既に存在する場合は、それを返します。
     */
    Table.prototype.createFoot = function() {

        var tfoot = this.table.tFoot;
        if (null == tfoot) {

            tfoot = this.table.createTFoot();
        }
        return tfoot;
    };

    /**
     * このテーブルにキャプションを設定します。
     * 既に存在する場合、上書きします。
     * @param captionString : 文字列 : キャプションとして表示する文字列
     */
    Table.prototype.setCaption = function(captionString) {

        if (null == captionString || 0 == captionString.length) {

            /*
             * nullや空文字の場合は何もしない。
             */
            return;
        }

        /*
         * 一旦キャプションを削除してから追加をする。
         */
        this.table.deleteCaption();
        var caption = this.table.createCaption();
        var textValueNode = document.createTextNode(captionString);
        caption.appendChild(textValueNode);
    };

    /**
     * 指定のインデックスのCOL要素を取得します。
     * 存在しない場合は作成します。
     */
    Table.prototype.getColAt = function(columnIndex) {

        var nodeList = this.table.getElementsByTagName("COL");
        if (columnIndex < nodeList.length) {

            /*
             * 指定のCOL要素が存在する。
             */
            return nodeList.item(columnIndex);

        } else {

            /*
             * 指定のCOL要素は存在しない。
             * この場合は、thead・tfoot・tbodyいずれかの存在するものの前に追加する。
             */
            var afterChild = null;
            if (null != this.table.tHead) {

                afterChild = this.table.tHead;

            } else if (null != this.table.tFoot) {

                afterChild = this.table.tFoot;

            } else if (0 != this.table.tBodies.length) {

                afterChild = this.table.tBodies.item(0);

            } else {

                /*
                 * 何もなければ、最後に追加する。
                 */
            }

            var rtnValue = null;
            var initCount = nodeList.length - 1;
            for (var i = initCount; i < columnIndex; i ++) {

                var col = document.createElement("col");
                rtnValue = this.table.insertBefore(col, afterChild);
            }

            return rtnValue;
        }
    };

    /**
     * テーブルの総行数(ヘッダ・ボディ・フッタ含む)を返します。
     * @return テーブルの総行数(ヘッダ・ボディ・フッタ含む)
     */
    Table.prototype.getRowCount = function() {

        return this.table.rows.length;
    };

    /**
     * テーブルのヘッダ部の合計行数を返します。
     * @return テーブルのボディ部の合計行数
     */
    Table.prototype.getHeadRowCount = function() {

        if (null == this.table.tHead) {

            return 0;
        }

        return this.table.tHead.rows.length;
    };

    /**
     * テーブルのヘッダ列のサイズを返します。
     * ヘッダ列の定義は、col要素のclassに"com_yscjp_htmlclass_headcolumn"が指定されている列です。
     */
    Table.prototype.getHeadColumnCount = function() {

        var headSize = 0;
        var nodeList = this.table.getElementsByTagName("COL");
        for (var i = 0; i < nodeList.length; i ++) {

            var col = nodeList.item(i);
            if ("com_yscjp_htmlclass_headcolumn" == col.className) {

                headSize ++;
            }
        }

        return headSize;
    };

    /**
     * テーブルのボディ部の合計行数を返します。
     * @return テーブルのボディ部の合計行数
     */
    Table.prototype.getBodiesRowCount = function() {

        var bodies = this.table.tBodies;
        var length = bodies.length;
        var count = 0;
        for (var i = 0; i < length; i ++) {

            count += bodies.item(i).rows.length;
        }
        return count;
    };

    /**
     * 指定インデックスの行エレメントを返します。
     * @param rowIndex : 数値 : 取得したい行の0を先頭とするテーブル全体の行数。
     * @return 行エレメント。該当する行が存在しない場合、null。
     */
    Table.prototype.getRowAt = function(rowIndex) {

        if (rowIndex >= this.table.rows.length) {

            return null;
        }

        return this.table.rows.item(rowIndex);
    };

    /**
     * 指定インデックスの行エレメントを、ボディ部から取得し返します。
     * @param rowIndex : 数値 : 取得したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、ボディ部全体の行数を指定する。
     * @return 行エレメント。該当する行が存在しない場合、null。
     */
    Table.prototype.getRowFromHeadAt = function(rowIndex) {

        /*
         * 引数のインデックスが、0より小さい場合、返す行はない。
         */
        if (0 > rowIndex) {

            return null;
        }

        if (null == this.table.tHead) {

            return null;
        }

        if (rowIndex >= this.table.tHead.rows.length) {

            return null;
        }

        return this.table.tHead.rows.item(rowIndex);
    };

    /**
     * 指定インデックスの行エレメントを、ボディ部から取得し返します。
     * @param rowIndex : 数値 : 取得したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、ボディ部全体の行数を指定する。
     * @return 行エレメント。該当する行が存在しない場合、null。
     */
    Table.prototype.getRowFromBodiesAt = function(rowIndex) {

        /*
         * 引数のインデックスが、0より小さい場合、返す行はない。
         */
        if (0 > rowIndex) {

            return null;
        }

        var index = rowIndex;
        var bodies = this.table.tBodies;
        var bodiesLength = bodies.length;
        for (var i = 0; i < bodiesLength; i ++) {

            var body = bodies.item(i);
            var rowLength = body.rows.length;
            if (index < rowLength) {
                /*
                 * このボディに該当行あり
                 */
                return body.rows.item(index);
            }
            else {
                /*
                 * このボディには該当行なし
                 */
                index -= rowLength;
            }
        }
        /*
         * ここへ来てしまったら、引数不正
         * ボディにある総行数よりも、引数の方が大きい
         */
        return null;
    };

    /**
     * 指定インデックスのセルを、ボディ部から取得し返します。
     * @param rowIndex : 数値 : 取得したいセルの0を先頭とする行数。
     *        テーブル全体の行数ではなく、ボディ部全体の行数を指定する。
     * @param columnIndex : 数値 : 取得したいセルの0を先頭とする列数。
     * @return 行インデックスと列インデックスに該当するセル要素。
     *         該当する行または列が存在しない場合、null。
     */
    Table.prototype.getCellFromBodiesAt = function (rowIndex, columnIndex) {

        var row = this.getRowFromBodiesAt(rowIndex);

        if (! row || row.cells.length <= columnIndex) {

            return null;
        }

        return row.cells.item(columnIndex);
    };

    /**
     * 指定のセルの右側のセルを返します。
     * 指定のセルの右側にセルが存在しない場合、nullを返します。
     */
    Table.prototype.getRightCell = function (cell) {

        var columnIndex = cell.cellIndex;
        var rowElement = cell.parentNode;
        if (columnIndex + 1 >= rowElement.cells.length) {
            return null;
        }
        return rowElement.cells.item(columnIndex + 1);
    };

    /**
     * 指定のセルの左側のセルを返します。
     * 指定のセルの左側にセルが存在しない場合、nullを返します。
     */
    Table.prototype.getLeftCell = function (cell) {

        var columnIndex = cell.cellIndex;
        if (0 == columnIndex) {
            return null;
        }
        return cell.parentNode.cells.item(columnIndex - 1);
    };

    /**
     * 指定のセルの上のセルを返します。
     * 指定のセルの上にセルが存在しない場合、nullを返します。
     */
    Table.prototype.getAboveCellFromSection = function (cell) {

        return this.getCellFromSection(cell, true);
    };

    /**
     * 指定のセルの下のセルを返します。
     * 指定のセルの下にセルが存在しない場合、nullを返します。
     */
    Table.prototype.getBelowCellFromSection = function (cell) {

        return this.getCellFromSection(cell, false);
    };

    /**
     * 指定のセルの上下のセルを返します。
     * isAbove が true の場合上のセル、false の場合下のセルを返します。
     */
    Table.prototype.getCellFromSection = function (cell, isAbove) {

        var columnIndex = cell.cellIndex;
        var rowElement = cell.parentNode;
        var rowIndex = rowElement.sectionRowIndex;
        var offset;

        if (isAbove) {

            if (0 == rowIndex) {

                return null;
            }

            offset = -1;

        } else {

            var rowCount = rowElement.parentNode.rows.length;
            if (rowIndex + 1 >= rowCount) {

                return null;
            }

            offset = 1;
        }

        var targetRow = rowElement.parentNode.rows.item(rowIndex + offset);
        if (columnIndex < targetRow.cells.length) {

            return targetRow.cells.item(columnIndex);
        }

        return null;
    };

    /**
     * 指定インデックスのセルを、ヘッダ部から取得し返します。
     * @param rowIndex : 数値 : 取得したいセルの0を先頭とする行数。
     *        テーブル全体の行数ではなく、フッタ部の行数を指定する。
     * @param columnIndex : 数値 : 取得したいセルの0を先頭とする列数。
     * @return 行インデックスと列インデックスに該当するセル要素。
     *         該当する行または列が存在しない場合、null。
     */
    Table.prototype.getCellFromHeadAt = function (rowIndex, columnIndex) {

        if (null == this.table.tHead) {

            return null;
        }

        if (rowIndex >= this.table.tHead.rows.length) {

            return null;
        }

        var row = this.table.tHead.rows.item(rowIndex);

        if (! row || row.cells.length <= columnIndex) {

            return null;
        }

        return row.cells.item(columnIndex);
    };

    /**
     * 指定インデックスのセルを、フッタ部から取得し返します。
     * @param rowIndex : 数値 : 取得したいセルの0を先頭とする行数。
     *        テーブル全体の行数ではなく、フッタ部の行数を指定する。
     * @param columnIndex : 数値 : 取得したいセルの0を先頭とする列数。
     * @return 行インデックスと列インデックスに該当するセル要素。
     *         該当する行または列が存在しない場合、null。
     */
    Table.prototype.getCellFromFootAt = function (rowIndex, columnIndex) {

        if (null == this.table.tFoot) {

            return null;
        }

        if (rowIndex >= this.table.tFoot.rows.length) {

            return null;
        }

        var row = this.table.tFoot.rows.item(rowIndex);

        if (! row || row.cells.length <= columnIndex) {

            return null;
        }

        return row.cells.item(columnIndex);
    };

    /**
     * 指定の行インデックスと列インッデクスが、有効かどうかをテストします。
     * 有効な場合true、無効な場合はfalseを返します。
     */
    Table.prototype.isValidIndex = function(rowIndex, columnIndex) {

        try {

            if (rowIndex < 0) {

                throw "com.yscjp.jsapp.html.Table#isValidIndex:Illegal argment rowIndex negative:" + rowIndex;
            }

            if (columnIndex < 0) {

                throw "com.yscjp.jsapp.html.Table#isValidIndex:Illegal argment columnIndex negative:" + columnIndex;
            }

            var rowItem = this.getRowFromBodiesAt(rowIndex);
            if (null == rowItem) {

                return false;
            }

            if (columnIndex < rowItem.cells.length) {

                return true;
            }
        }
        catch (e) {}

        return false;
    };

    /**
     * ボディ部の指定のインデックスに行を挿入する。
     * インデックスが0より小さい場合や、ボディ部の総行数より大きい場合は、
     * ボディ部の最期に追加する。
     * @param rowIndex : 数値 : 追加／挿入したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、ボディ部全体の行数を指定する。
     * @return 追加／挿入した行要素。
     */
    Table.prototype.insertRowAt = function(rowIndex) {

        var bodiesRowCount = this.getBodiesRowCount();

        /*
         * 引数のインデックスが、0より小さい場合や、ボディ部の総行数より
         * 大きい場合は、最期に追加する。
         */
        if (0 > rowIndex || bodiesRowCount <= rowIndex) {

            rowIndex = bodiesRowCount;
        }

        var body = null;
        var index = rowIndex;
        var bodies = this.table.tBodies;
        var bodiesLength = bodies.length;
        for (var i = 0; i < bodiesLength; i ++) {

            body = bodies.item(i);
            var rowLength = body.rows.length;
            if (index < rowLength) {
                /*
                 * このボディに該当行あり
                 */
                return body.insertRow(index);
            }
            else {
                /*
                 * このボディには該当行なし
                 */
                index -= rowLength;
            }
        }

        /*
         * 最期のボディ部に行を追加する。
         */
        if (null != body) {

            return body.insertRow(-1);
        }

        /*
         * ここへくることは、ロジック上あり得ない。
         */
        return null;
    };


    /**
     * ヘッダ部の指定のインデックスに行を挿入する。
     * インデックスが0より小さい場合や、ヘッダ部の総行数より大きい場合は、ヘッダ部の最後に追加する。
     * @param rowIndex : 数値 : 追加／挿入したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、ヘッダ部全体の行数を指定する。
     * @return 追加／挿入した行要素。
     */
    Table.prototype.insertHeadRowAt = function(rowIndex) {

        var head = this.createHead();
        return this.insertSectionRowAt(rowIndex, head);
    };


    /**
     * フッタ部の指定のインデックスに行を挿入する。
     * インデックスが0より小さい場合や、フッタ部の総行数より大きい場合は、フッタ部の最後に追加する。
     * @param rowIndex : 数値 : 追加／挿入したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、フッタ部全体の行数を指定する。
     * @return 追加／挿入した行要素。
     */
    Table.prototype.insertFootRowAt = function(rowIndex) {

        var foot = this.createFoot();
        return this.insertSectionRowAt(rowIndex, foot);
    };


    /**
     * セクションの指定のインデックスに行を挿入する。
     * インデックスが0より小さい場合や、セクションの総行数より大きい場合は、セクションの最後に追加する。
     * @param rowIndex : 数値 : 追加／挿入したい行の0を先頭とする行数。
     *        テーブル全体の行数ではなく、セクションの行数を指定する。
     * @return 追加／挿入した行要素。
     */
    Table.prototype.insertSectionRowAt = function(rowIndex, section) {

        var sectionRowCount = section.rows.length;

        /*
         * 引数のインデックスが、0より小さい場合や、セクションの総行数より
         * 大きい場合は、最後に追加する。
         */
        if (0 > rowIndex || sectionRowCount <= rowIndex) {

            rowIndex = sectionRowCount;
        }

        return section.insertRow(rowIndex);
    };

    Table.prototype.removeHeadRowAt = function(rowIndex) {

        var thead = this.table.tHead;
        this.removeSectionRowAt(rowIndex, thead);
    };
    Table.prototype.removeFootRowAt = function(rowIndex) {

        var tfoot = this.table.tFoot;
        this.removeSectionRowAt(rowIndex, tfoot);
    };
    /**
     * 指定のセクションから、指定の行を削除します。
     */
    Table.prototype.removeSectionRowAt = function(rowIndex, section) {

        if (null == section) {

            /*
             * セクションが無ければ、何もしない。
             */
            return;
        }

        var rowCount = section.rows.length();
        if (rowCount <= rowIndex) {

            /*
             * 指定の行が存在しない場合は、なにもしない。
             */
            return;
        }

        section.deleteRow(rowIndex);
    };

    /**
     * 全ての行を削除します。
     */
    Table.prototype.removeAllRow = function() {

        /*
         * theadとtfootを削除します。
         */
        this.table.deleteTHead();
        this.table.deleteTFoot();

        var rowCount = this.getRowCount();
        for (var i = 0; i < rowCount; i ++) {

            this.table.deleteRow(-1);
        }
    };
})();
