/**
 * Webアプリケーション用のツール類.
 *
 * Version 00.10.00.00
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2011-2012 Yagyu Teru, http://yscjp.com/
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.tools) com.yscjp.jsapp.tools = {};


/**
 * XML Load & Save の実装。
 * XMLHttpRequestにより、実現しています。
 */
com.yscjp.jsapp.tools.XmlLS = function () {

    this.httpRequest = com.yscjp.jsapp.util.XmlUtil.createXMLHttpRequest();
    this.resultCode = -1;
};

(function() {

    var Log = com.yscjp.jsapp.util.Log;
    var XmlUtil = com.yscjp.jsapp.util.XmlUtil;
    var XmlLS = com.yscjp.jsapp.tools.XmlLS;

    /**
     * URIから、テキストドキュメントを、同期的に読み込む。
     * @param uri 読み込むxmlドキュメントのURI
     *            HTTPのGETメソッドでアクセスできる対象でなければならない。
     * @return 指定のURIを読み込んだ結果のドキュメント。
     *         読み込みのエラーが発生した場合、nullを返す。
     *         エラーの詳細は、XmlLS#resultCodeと、
     *         XmlLS#resultTextを参照。
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.getTextFromURI = function (uri) {

        this.resultCode = -1;
        this.resultText = "Not Send";

        this.httpRequest.open("GET", uri, false);
        this.httpRequest.send(null);
        this.resultCode = this.httpRequest.status;
        this.resultText = this.httpRequest.responseText;

        if (200 != this.resultCode) {

            Log.debug(this.resultCode + " : " + this.resultText);
        }
        return this.httpRequest.responseText;
    };

    /**
     * URIからの、XMLドキュメント同期読み込み.
     * @param uri 読み込むxmlドキュメントのURI
     *            HTTPのGETメソッドでアクセスできる対象でなければならない。
     * @return 指定のURIを読み込んだ結果のドキュメント。
     *         読み込みのエラーが発生した場合や、読み込んだデータのMIMEタイプが、
     *         xmlドキュメントにパースできない場合、nullを返す。
     *         エラーの詳細は、XmlLS#resultCodeと、
     *         XmlLS#resultTextを参照。
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.perseFromURI = function (uri) {

        return this.syncHttpRequest("GET", "", uri);
    };

    /**
     * URIへの、XMLドキュメント書き込み
     * @param savedValue セーブするテキストまたはドキュメント
     * @param uri 読み込むxmlドキュメントのURI
     *            HTTPのGETメソッドでアクセスできる対象でなければならない。
     * @return 指定のURIを読み込んだ結果のドキュメント。
     *         読み込みのエラーが発生した場合や、読み込んだデータのMIMEタイプが、
     *         xmlドキュメントにパースできない場合、nullを返す。
     *         エラーの詳細は、XmlLS#resultCodeと、
     *         XmlLS#resultTextを参照。
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.writeToURI = function (savedValue, uri) {

        return this.syncHttpRequest("POST",
                                    XmlUtil.writeToString(savedValue),
                                    uri);
    };

    /**
     * httpリクエストの同期発行
     * @param method GET/POST
     * @param savedValue セーブするテキストまたはドキュメント。
     * @param uri 読み込むxmlドキュメントのURI
     *            HTTPのGETメソッドでアクセスできる対象でなければならない。
     * @return 指定のURIを読み込んだ結果のドキュメント。
     *         読み込みのエラーが発生した場合や、読み込んだデータのMIMEタイプが、
     *         xmlドキュメントにパースできない場合、nullを返す。
     *         エラーの詳細は、XmlLS#resultCodeと、
     *         XmlLS#resultTextを参照。
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.syncHttpRequest = function (method, savedValue, uri) {

        this.resultCode = -1;
        this.resultText = "Not Send";

        this.httpRequest.open(method, uri, false);
        this.httpRequest.setRequestHeader("Content-Type", "text/xml");
//            AppUtil.printLog("savedValue(" + savedValue.xml + ")");
        this.httpRequest.send(savedValue);
        this.resultCode = this.httpRequest.status;
        this.resultText = this.httpRequest.responseText;

        if (200 != this.resultCode) {

            Log.debug(this.resultCode + " : " + this.resultText);
        }
        return XmlUtil.modifyDocument(this.httpRequest.responseXML);
    };

    /**
     * URIからの、XMLドキュメント非同期読み込み
     * @param uri 読み込むxmlドキュメントのURI
     *            HTTPのGETメソッドでアクセスできる対象でなければならない。
     * @param listener ロードイベントのリスナクラス。
     *
     * @return 指定のURIを読み込んだ結果のドキュメント。
     *         読み込みのエラーが発生した場合や、読み込んだデータのMIMEタイプが、
     *         xmlドキュメントにパースできない場合、nullを返す。
     *         エラーの詳細は、XmlLS#resultCodeと、
     *         XmlLS#resultTextを参照。
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.perseAsyncFromURI = function (uri, listener) {

        this.resultCode = -1;
        this.resultText = "Not Send";

        var httpRequest = this.httpRequest;
        var handler = function(evnt) {

            var d = new Date();
            Log.debug(d.toString() + "." + d.getMilliseconds() + ":" + httpRequest.readyState);

            if (4 == httpRequest.readyState) {

                Log.debug("status:" + httpRequest.status + "/" + httpRequest.responseXML);

                if (200 == httpRequest.status &&
                    null != httpRequest.responseXML) {

                    var xmlDoc = com.yscjp.jsapp.util.XmlUtil.modifyDocument(httpRequest.responseXML);
                    listener.documentLoaded(xmlDoc);
                }
            }
        };
        this.httpRequest.onreadystatechange = handler;

        this.httpRequest.open("GET", uri, true);
        this.httpRequest.send("");
    };

    /**
     * HTTPの結果コードを返します.
     * @return HTTPの結果コード
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.getResultCode = function () {

        this.resultCode;
    };

    /**
     * HTTPの結果コードを返します.
     * @return HTTPの結果コード
     * @memberOf com.yscjp.jsapp.tools.XmlLS
     */
    XmlLS.prototype.getResultText = function () {

        this.httpRequest.resultText;
    };


//     XmlLS.httpHandler = function (evnt) {
//
//         if (4 == httpRequest.readyState) {
//
//             if (200 == httpRequest.status &&
//                 null != httpRequest.responseXML) {
//
//                 var xmlDoc = XmlUtil.modifyDocument(httpRequest.responseXML);
//                 listener.documentLoaded(xmlDoc);
//             }
//         }
//     }
 })();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.tools.XmlLS,
                                       "com.yscjp.jsapp.tools.XmlLS");


com.yscjp.jsapp.tools.HtmlPoint = function (x, y) {

    this.x = x;
    this.y = y;
};

com.yscjp.jsapp.tools.HtmlPoint.prototype.equals = function (point) {
    if (this.x == point.x && this.y == point.y) {
        return true;
    } else {
        return false;
    }
};

com.yscjp.jsapp.tools.HtmlPoint.prototype.toString = function () {
    return "com.yscjp.jsapp.tools.HtmlPoint [x:" + this.x + " y:" + this.y + "]";
};

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.tools.HtmlPoint,
                                       "com.yscjp.jsapp.tools.HtmlPoint");

/**
 * @class
 */
com.yscjp.jsapp.tools.HtmlPosition = function () {};

(function() {

    var Log = com.yscjp.jsapp.util.Log;
    var CSSView = com.yscjp.jsapp.util.CSSView;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var HtmlPosition = com.yscjp.jsapp.tools.HtmlPosition;
    var HtmlPoint = com.yscjp.jsapp.tools.HtmlPoint;

    /**
     * @author bison
     *
     */
    HtmlPosition.createHtmlPosition = function (element) {

        htmlPosition = new HtmlPosition();
        htmlPosition.setElement(element);
        htmlPosition.updatePosition();

        return htmlPosition;
    };

    HtmlPosition.prototype.setElement = function (element) {

        this.element = element;
    };

    HtmlPosition.prototype.updatePosition = function () {

        var offsetTop;
        var offsetLeft;
        var topPotision;
        var leftPotision;
        var height = this.element.offsetHeight;
        var width = this.element.offsetWidth;
        var scrollTop = 0;
        var scrollLeft = 0;

        var cssPosition = CSSView.getElementStyle(this.element, "position");
        if ("fixed" == cssPosition) {

            offsetTop = this.element.offsetTop;
            offsetLeft = this.element.offsetLeft;
            topPotision = this.element.offsetTop;
            leftPotision = this.element.offsetLeft;

        } else {

            var parentNode = this.element.parentNode;
            while (null != parentNode) {

                if (parentNode.scrollTop) {

                    scrollTop += parentNode.scrollTop;
                }

                if (parentNode.scrollLeft) {

                    scrollLeft += parentNode.scrollLeft;
                }

                //Log.debug("---- scrollTop:"+scrollTop+" ----");
                //Log.debug("---- scrollLeft:"+scrollLeft+" ----");

                parentNode = parentNode.parentNode;
            }

            offsetTop = JSHtmlUtil.getTopPixel(this.element);
            offsetLeft = JSHtmlUtil.getLeftPixel(this.element);
            Log.debug("---- offsetTop:"+offsetTop+" ----");
            Log.debug("---- offsetLeft:"+offsetLeft+" ----");

            topPotision = offsetTop - scrollTop;
            leftPotision = offsetLeft - scrollLeft;
        }

        this.topPx      = topPotision;
        this.leftPx     = leftPotision;
        this.heightPx   = height;
        this.widthPx    = width;
        this.offsetTop  = offsetTop;
        this.offsetLeft = offsetLeft;
        this.scrollTop  = scrollTop;
        this.scrollLeft = scrollLeft;
    };

    HtmlPosition.prototype.getTopOnClient = function () {
        return this.offsetTop;
    };

    HtmlPosition.prototype.getLeftOnClient = function () {
        return this.offsetLeft;
    };

    HtmlPosition.prototype.getTop = function () {
        return this.topPx;
    };

    HtmlPosition.prototype.getLeft = function () {
        return this.leftPx;
    };

    HtmlPosition.prototype.getHeight = function () {
        return this.heightPx;
    };

    HtmlPosition.prototype.getWidth = function () {
        return this.widthPx;
    };

    HtmlPosition.prototype.getPoint = function (direction) {

        switch (direction) {
        case "N" :
            return this.getN();
        case "NEN" :
            return this.getNEN();
        case "NE" :
            return this.getNE();
        case "ENE" :
            return this.getENE();
        case "E" :
            return this.getE();
        case "ESE" :
            return this.getESE();
        case "SE" :
            return this.getSE();
        case "SES" :
            return this.getSES();
        case "S" :
            return this.getS();
        case "SWS" :
            return this.getSWS();
        case "SW" :
            return this.getSW();
        case "WSW" :
            return this.getWSW();
        case "W" :
            return this.getW();
        case "WNW" :
            return this.getWNW();
        case "NW" :
            return this.getNW();
        case "NWN" :
            return this.getNWN();
        default :
            return this.getCenter();
        }
    };

    /**
     * 要素の中心のポイントを返します.
     * 以下の図の、(1)のポイントとなります。
     * <pre>
     *      (16)  (17) (2)  (3)  (4)
     *       NW   NWN   N   NEN   NE
     *        +----+----+----+----+
     *        |                   |
     * (15)WNW+                   +ENE(5)
     *        |        (1)        |
     *   (14)W+         +         +E(6)
     *        |                   |
     * (13)WSW+                   +ESE(7)
     *        |                   |
     *        +----+----+----+----+
     *       SW   SWS   S   SES   SE
     *      (12)  (11) (10) (9)  (8)
     * </pre>
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getCenter = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 2),
                              this.getTop() + (this.getHeight() / 2));
    };

    HtmlPosition.prototype.getCenterOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 2),
                              this.getTopOnClient() + (this.getHeight() / 2));
    };

    /**
     * 要素の北(N)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getN = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 2), this.getTop());
    };

    HtmlPosition.prototype.getNOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 2), this.getTopOnClient());
    };

    /**
     * 要素の北北東(NEN)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getNEN = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 4 * 3), this.getTop());
    };

    HtmlPosition.prototype.getNENOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 4 * 3), this.getTopOnClient());
    };

    /**
     * 要素の北東(NE)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getNE = function () {

        return new HtmlPoint(this.getLeft() + this.getWidth(), this.getTop());
    };

    HtmlPosition.prototype.getNEOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + this.getWidth(), this.getTopOnClient());
    };

    /**
     * 要素の東北東(ENE)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getENE = function () {

        return new HtmlPoint(this.getLeft() + this.getWidth(),
                              this.getTop() + (this.getHeight() / 4));
    };

    HtmlPosition.prototype.getENEOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + this.getWidth(),
                              this.getTopOnClient() + (this.getHeight() / 4));
    };

    /**
     * 要素の東(E)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getE = function () {

        return new HtmlPoint(this.getLeft() + this.getWidth(),
                              this.getTop() + (this.getHeight() / 2));
    };

    HtmlPosition.prototype.getEOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + this.getWidth(),
                              this.getTopOnClient() + (this.getHeight() / 2));
    };

    /**
     * 要素の東南東(ESE)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getESE = function () {

        return new HtmlPoint(this.getLeft() + this.getWidth(),
                              this.getTop() + (this.getHeight() / 4 * 3));
    };

    HtmlPosition.prototype.getESEOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + this.getWidth(),
                              this.getTopOnClient() + (this.getHeight() / 4 * 3));
    };

    /**
     * 要素の南東(SE)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getSE = function () {

        return new HtmlPoint(this.getLeft() + this.getWidth(),
                              this.getTop() + this.getHeight());
    };

    HtmlPosition.prototype.getSEOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + this.getWidth(),
                              this.getTopOnClient() + this.getHeight());
    };

    /**
     * 要素の南南東(SES)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getSES = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 4 * 3),
                              this.getTop() + this.getHeight());
    };

    HtmlPosition.prototype.getSESOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 4 * 3),
                              this.getTopOnClient() + this.getHeight());
    };

    /**
     * 要素の南(S)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getS = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 2),
                              this.getTop() + this.getHeight());
    };

    HtmlPosition.prototype.getSOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 2),
                              this.getTopOnClient() + this.getHeight());
    };

    /**
     * 要素の南南西(SWS)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getSWS = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 4),
                              this.getTop() + this.getHeight());
    };

    HtmlPosition.prototype.getSWSOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 4),
                              this.getTopOnClient() + this.getHeight());
    };

    /**
     * 要素の南西(SW)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getSW = function () {

        return new HtmlPoint(this.getLeft(), this.getTop() + this.getHeight());
    };

    HtmlPosition.prototype.getSWOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient(), this.getTopOnClient() + this.getHeight());
    };

    /**
     * 要素の西南西(WSW)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getWSW = function () {

        return new HtmlPoint(this.getLeft(),
                              this.getTop() + (this.getHeight() / 4 * 3));
    };

    HtmlPosition.prototype.getWSWOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient(),
                              this.getTopOnClient() + (this.getHeight() / 4 * 3));
    };

    /**
     * 要素の西(W)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getW = function () {

        return new HtmlPoint(this.getLeft(),
                              this.getTop() + (this.getHeight() / 2));
    };

    HtmlPosition.prototype.getWOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient(),
                              this.getTopOnClient() + (this.getHeight() / 2));
    };

    /**
     * 要素の西北西(WNW)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getWNW = function () {

        return new HtmlPoint(this.getLeft(),
                              this.getTop() + (this.getHeight() / 4));
    };

    HtmlPosition.prototype.getWNWOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient(),
                              this.getTopOnClient() + (this.getHeight() / 4));
    };

    /**
     * 要素の北西(NW)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getNW = function () {

        return new HtmlPoint(this.getLeft(), this.getTop());
    };

    HtmlPosition.prototype.getNWOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient(), this.getTopOnClient());
    };

    /**
     * 要素の北北西(NWN)のポイントを返します.
     * getCenterの説明にある図の、(2)のポイントとなります。
     * @return 要素の中心のポイント
     * @memberOf com.yscjp.jsapp.tools.HtmlPosition
     */
    HtmlPosition.prototype.getNWN = function () {

        return new HtmlPoint(this.getLeft() + (this.getWidth() / 4), this.getTop());
    };

    HtmlPosition.prototype.getNWNOnClient = function () {

        return new HtmlPoint(this.getLeftOnClient() + (this.getWidth() / 4), this.getTopOnClient());
    };

    HtmlPosition.prototype.toString = function () {
        return "com.yscjp.jsapp.tools.HtmlPosition [Top:" + this.getTop() +
                " Left:" + this.getLeft() +
                " Height:" + this.getHeight() +
                " Width:" + this.getWidth() + "]";
    };

    HtmlPosition.prototype.moveOffset = function (offsetTop, offsetLeft) {

        var top = this.offsetTop + offsetTop;
        var left = this.offsetLeft + offsetLeft;

        HtmlPosition.moveTo(this.element, top, left);
    };

    HtmlPosition.prototype.moveTo = function (offsetTop, offsetLeft) {

        HtmlPosition.moveTo(this.element, offsetTop, offsetLeft);
    };

    HtmlPosition.moveTo = function (element, top, left) {

//        var styleTop = CSSView.getStyleTop(element);
//        var styleLeft = CSSView.getStyleLeft(element);
//        var currentTop = element.offsetTop;
//        var currentLeft = element.offsetLeft;
//
//        var diffTop = top - currentTop;
//        var diffLeft = left - currentLeft;
//
//        element.style.top = (styleTop + diffTop) + 'px';
//        element.style.left = (styleLeft + diffLeft) + 'px';

        element.style.top = top + 'px';
        element.style.left = left + 'px';
    };

    HtmlPosition.resizeBase = function (element, referencePoint, height, width) {

        if (0 == height || 0 == width) {

            return;
        }

        switch (referencePoint) {

            case "NW" : // 基準点が北西（左上）

                /*
                 * 位置はそのままでサイズのみ変更する。
                 */
                HtmlPosition.resize(element, height, width);
                break;

            case "NE" : // 基準点が北東（右上）

                /*
                 * 横幅の差分分左に移動して、サイズを変更する。
                 */
                HtmlPosition.moveTo(element,
                                    element.offsetTop,
                                    element.offsetLeft + element.offsetWidth - width);
                HtmlPosition.resize(element, height, width);
                break;

            case "SE" : // 基準点が南東（右下）

                /*
                 * 横幅の差分分左に移動し、高さの差分分上に移動し、サイズを変更する。
                 */
                HtmlPosition.moveTo(element,
                                    element.offsetTop + element.offsetHeight - height,
                                    element.offsetLeft + element.offsetWidth - width);
                HtmlPosition.resize(element, height, width);
                break;

            case "SW" : // 基準点が南西（左下）

                /*
                 * 高さの差分分上に移動し、サイズを変更する。
                 */
                HtmlPosition.moveTo(element,
                                    element.offsetTop + element.offsetHeight - height,
                                    element.offsetLeft);
                HtmlPosition.resize(element, height, width);
                break;

            default :
        }
    };

    HtmlPosition.resize = function (element, height, width) {

        var styleWidth = CSSView.getStyleWidth(element);
        var styleHeight = CSSView.getStyleHeight(element);
        var currentWidth = element.offsetWidth;
        var currentHeight = element.offsetHeight;

        if (null == styleWidth) {

            styleWidth = currentWidth;
        }

        if (null == styleHeight) {

            styleHeight = currentHeight;
        }

        var diffWidth = width - currentWidth;
        var diffHeight = height - currentHeight;

        element.style.width = (styleWidth + diffWidth) + 'px';
        element.style.height = (styleHeight + diffHeight) + 'px';
    };

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.tools.HtmlPosition,
                                       "com.yscjp.jsapp.tools.HtmlPosition");


com.yscjp.jsapp.tools.WindowSurveillance = function (win, delayTime, onloadHdl, oncloseHdl, targetWindow, prefix) {

    this.win = win;
    this.delayTime = delayTime;
    this.onloadHdl = onloadHdl;
    this.oncloseHdl = oncloseHdl;
    this.newBody = null;
    this.timer = new com.yscjp.jsapp.tools.JSTimer(this, this.delayTime, true, targetWindow, prefix);
    this.errorCount = 0;
    this.beforeListener = null;
};

(function() {

    var Log = com.yscjp.jsapp.util.Log;

    function handleEvent (e) {

        Log.debug("---------- com.yscjp.jsapp.tools.WindowSurveillance#handleEvent ----------");

        this.timer.start();
    }

    var action = function () {

        Log.debug("---------- com.yscjp.jsapp.tools.WindowSurveillance#action ----------");

        var codeLocation = "";

        try {

            codeLocation = "0001";
            if (!this.win) {

                return true;
            }

            codeLocation = "0002";
            if (this.win.closed) {

                if (this.oncloseHdl) {

                    setTimeout(this.oncloseHdl, 0);
                }
                return false;
            }

            codeLocation = "0003";
            if ("unknown" == typeof this.win.document || !this.win.document) {

                return true;
            }

            codeLocation = "0004";
            if (!this.win.document.body) {

                return true;
            }

            codeLocation = "0005";
            if (this.newBody != this.win.document.body) {

                codeLocation = "0006";
                if ('complete' != this.win.document.readyState) {

                    return true;
                }

                this.newBody = this.win.document.body;

                try {

                    codeLocation = "0007";
                    this.onloadHdl();

                } catch (e) {

                    Log.warning("this.onloadHdl:" + e.name + " " + e.message);
                    return false;
                }

                codeLocation = "0008";
                var isListenerAdded = false;
                if (this.win.addEventListener) {

                    this.win.addEventListener("unload", this, false);
                    isListenerAdded = true;

                } else {

                    codeLocation = "0009";
                    this.win.detachEvent("onunload", this.beforeListener);
                    var __timer__ = this.timer;
                    this.beforeListener = function () {
                        __timer__.start();
                    };
                    codeLocation = "0010";
                    this.win.attachEvent("onunload", this.beforeListener);
                    isListenerAdded = true;
                }

                if (isListenerAdded) {

                    this.timer.stop();
                }
            }

            this.errorCount = 0;

        } catch (e) {

            Log.warning("com.yscjp.jsapp.tools.WindowSurveillance#action:" + e.name + " " + e.message + " [" + this.errorCount + "] " + codeLocation);
            if (this.errorCount > 10) {

                return false;
            }
            this.errorCount ++;
        }

        return true;
    };

    function start() {

        Log.debug("---------- com.yscjp.jsapp.tools.WindowSurveillance#start ----------");

        this.timer.start();
    };

    com.yscjp.jsapp.tools.WindowSurveillance.prototype.handleEvent = handleEvent;
    com.yscjp.jsapp.tools.WindowSurveillance.prototype.action = action;
    com.yscjp.jsapp.tools.WindowSurveillance.prototype.start = start;

})();

/**
 * タイマークラスコンストラクタ.
 * @class オブジェクトをリスナとして登録できるタイマ。
 * @param オブジェクト : リスナオブジェクト。
 *                       必ず、actionメソッドを実装している必要がある。
 * @param 数値 : 遅延時間(ミリ秒)
 * @param boolean : 繰り返しリスナを呼び出すかの指定。
 *                  繰り返し呼び出す場合、trueそれ以外はfalseを指定する。
 */
com.yscjp.jsapp.tools.JSTimer = function (listener, delay, isRepeats, targetWindow, prefix) {

    this.init(listener, delay, isRepeats, targetWindow, prefix);
};
(function() {

    var JSLogicalException = com.yscjp.jsapp.exception.JSLogicalException;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var Log = com.yscjp.jsapp.util.Log;
    var JSTimer = com.yscjp.jsapp.tools.JSTimer;

    JSTimer.listenerList = new Array();
    JSTimer.timerList = new Array();
    JSTimer.listenerId = 1000;
    JSTimer.timerListener = function (listenerId) {

        var listener = JSTimer.listenerList[listenerId];
        var timer = JSTimer.timerList[listenerId];
        if (! listener) {

            Log.error("JSTimer.timerListener error("+listenerId+")");
            throw JSLogicalException.create(new Error(), "listener is null. " + listenerId);
        }
        if (! timer) {

            Log.error("JSTimer.timerListener error("+listenerId+")");
            throw JSNullPointerException.create(new Error(), "timer is null. " + listenerId);
        }

        var rtn = false;
        try {

            rtn = listener.action(timer);

        } finally {

            if (timer.isRepeats) {

                if (rtn) {

                    return;
                }
            }

            Log.info("JSTimer stoped.");
            timer.stop();
        }
    };

    /**
     * 初期化処理
     * @param listener {Object} 登録するリスナオブジェクト。必ず、action メソッドが定義されている必要があります。
     * @param delay {Number} ミリ秒単位の遅延時間
     * @param isRepeats {boolean} 繰り返し実行するかの指示。true の場合繰り返しリスナが呼び出されます。
     * @param targetWindow {Window} setInterval/setTimeout メソッドを呼び出すWindowを指定します。
     * google chrome や firefox の場合、アクティブではないタブやWindowでタイマを実行した場合、
     * 指定の間隔ではなく1000ミリ秒間隔で実行されてしまいます。
     * これを回避するために、アクティブなWindowでタイマを実行出来るようにするインターフェイスです。
     * @param prefix {string} targetWindow を指定した場合に、タイマで実行するタイマリスナが
     * 実装されているWindowを定義する文字列。
     * targetWindowが指定されていて、prefix が指定されていない場合は、"opener" をデフォルトとして設定します。
     * ここで指定された文字列と、"com.yscjp.jsapp.tools.JSTimer.timerListener(xxxx)" を、
     * '.'でつないた文字列を、タイマで実行します。
     * @memberOf com.yscjp.jsapp.tools.JSTimer
     */
    JSTimer.prototype.init = function (listener, delay, isRepeats, targetWindow, prefix) {

        ArgChk.notNull(listener);
        ArgChk.typeOf(listener.action, "function");
        delay = ArgChk.typeOf(delay, "number", 10);
        isRepeats = ArgChk.notUndefined(isRepeats, false);
        targetWindow = ArgChk.notUndefined(targetWindow, null);
        prefix = ArgChk.notUndefined(prefix, "opener");

        this.delay = delay;
        this.listener = listener;
        this.isRepeats = isRepeats;
        this.targetWindow = targetWindow;
        this.prefix = prefix;
        this.listenerId = JSTimer.listenerId;
        this.isStoped = true;
        JSTimer.listenerId ++;
    };

    /**
     * タイマ開始
     */
    JSTimer.prototype.start = function () {

        this.isStoped = false;
        JSTimer.listenerList[this.listenerId] = this.listener;
        JSTimer.timerList[this.listenerId] = this;

        var execStr = "com.yscjp.jsapp.tools.JSTimer.timerListener("+this.listenerId+")";
        var execWin = window;
        if (this.targetWindow) {

            execWin = this.targetWindow;
            execStr = this.prefix + "." + execStr;
        }

        if (this.isRepeats) {

            this.timerId = execWin.setInterval(execStr, this.delay);

        } else {

            this.timerId = execWin.setTimeout(execStr, this.delay);
        }
    };

    /**
     * タイマ終了
     * 繰り返さない指定の場合は、このメソッドをコールした後、
     * 再度startメソッドをコールすることで、このインスタンスに、
     * タイマを実行させることができる。
     */
    JSTimer.prototype.stop = function () {

        if (this.isStoped) {

            return;
        }
        this.isStoped = true;

        var execWin = window;
        if (this.targetWindow) {

            execWin = this.targetWindow;
        }

        if (this.isRepeats) {

            execWin.clearInterval(this.timerId);

        } else {

            execWin.clearTimeout(this.timerId);
        }

        JSTimer.listenerList.splice(this.listenerId, 1);
        JSTimer.timerList.splice(this.listenerId, 1);
    };
})();

