/**
 * Web自動化ツール.
 * Webアプリケーションを、JavaのRobotクラスが生成するマウスやキーボードのイベントで、
 * 自動的に動作させるツールです。
 * プレゼンテーションや自動テストに利用できます。
 * イベントキャプチャもサポートするので、一度実行した操作を繰り返すことも可能です。
 * Java Appletが動作する環境が必要です。
 *
 * Version 00.10.00.02
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2011-2012 Yagyu Teru, http://yscjp.com/
 *
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.autorun) com.yscjp.jsapp.autorun = {};

com.yscjp.jsapp.autorun.Common = function () {};
(function() {

})();

com.yscjp.jsapp.autorun.Autorun = function () {};
(function() {

    var JSEvent = com.yscjp.jsapp.JSEvent;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var Log = com.yscjp.jsapp.util.Log;
    var Autorun = com.yscjp.jsapp.autorun.Autorun;

    var executeDoc = null;
    Autorun.delayTime = 100;
    var loadListener = null;
    var unloadListener = null;
    var maxSurveillanceCount = 10;
    var surveillanceinterval = Autorun.delayTime;
    var isUseTargetwinTimer = false;
    var isOutStop = true;

    var targetWindow = null;
    var settingDelayTime;

    /*
     * 実行するシナリオを格納した配列
     */
    var execList = null;

    /*
     * 以前のバージョンとインターフェイスを合わせるために用意する。
     * 本来は、targetWindow が正しい。
     */
    var newWindow = null;
    var index = 0;
    var isNonSetTimeout = false;

    var beforBody = null;
    var waitCount = 0;

    /*
     * このフラグにtrueを設定すると、自動実行が中断されます。
     */
    var isStop = false;

    /*
     * プライベートメソッドを管理する内部クラス。
     * トレースを取得する為に、一元管理していつ必要があるため、このような実装方法としている。
     */
    var __PFunc = function () {};

    /*
     * repeatFromIndex や repeatFromId メソッドでリピートする回数。
     */
    var repeatCount = -1;

    /*
     * 終了処理
     */
    var finallyExec = null;

    /**
     * Autorun の初期化を行います.
     * 各パラメータの値を null とした場合、直前に設定した値が有功となります。
     * @param {org.xml.Document} _executeDoc 実行するシナリオを記述したXMLドキュメント。
     * このパラメータにデフォルトの値は有りません。
     * @param {Number} _delayTime シナリオを実行するミリ秒単位の間隔。
     * このパラメータのデフォルトは、100です。
     * @param {Function} _loadListener ドキュメントのロード時に呼び出されるリスナ。
     * このパラメータにデフォルトの値は有りません。
     * @param {Function} _unloadListener ドキュメントのアンロード時に呼び出されるリスナ
     * このパラメータにデフォルトの値は有りません。
     * @param {Number} _surveillanceinterval ドキュメントのロード判定をする間隔をミリ秒単位で指定します。
     * このパラメータのデフォルトは、_delayTimeと同じです。
     * 実際の待ち合わせ時間は、約 _surveillanceinterval × _maxSurveillanceCount ミリ秒となります。
     * @param {Number} _maxSurveillanceCount ドキュメントのロードを待つ回数を指定します。
     * このパラメータのデフォルトは、10です。
     * 実際の待ち合わせ時間は、約 _surveillanceinterval × _maxSurveillanceCount ミリ秒となります。
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function initialize (_executeDoc,
                         _delayTime,
                         _loadListener,
                         _unloadListener,
                         _surveillanceinterval,
                         _maxSurveillanceCount,
                         _isUseTargetwinTimer,
                         _isOutStop) {

        executeDoc = ArgChk.notNull(_executeDoc, executeDoc);
        Autorun.delayTime = ArgChk.notNull(_delayTime, Autorun.delayTime);
        loadListener = ArgChk.notNull(_loadListener, loadListener);
        unloadListener = ArgChk.notNull(_unloadListener, unloadListener);
        maxSurveillanceCount = ArgChk.notNull(_maxSurveillanceCount, maxSurveillanceCount);
        surveillanceinterval = ArgChk.notNull(_surveillanceinterval, Autorun.delayTime);
        isUseTargetwinTimer = ArgChk.notNull(_isUseTargetwinTimer, false);
        isOutStop = ArgChk.notNull(_isOutStop, true);

        settingDelayTime = Autorun.delayTime;
    };

    /**
     * 初期化で指定されたシナリオを実行します.
     * 必ず、事前にinitializeが実行されている必要が有ります。
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function run () {

        ArgChk.notNull(executeDoc);
//        ArgChk.notNull(loadListener);
//        ArgChk.notNull(unloadListener);

        var execs = executeDoc.getElementsByTagName("exec");
        if (null == execs || 0 == execs.length) {

            Log.error("exec element not found.");
            return;
        }
        execList = new Array();
        for (var i = 0; i < execs.length; i++) {
            execList.push(execs[i]);
        }

        finallyExec = null;
        var finallyElements = executeDoc.getElementsByTagName("finally");
        if (0 != finallyElements.length) {

            finallyExec = __PFunc.getExecString(finallyElements[0]);
        }

        index = 0;
        isNonSetTimeout = false;
        isStop = false;
        repeatCount = -1;
        setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
    };

    /**
     * シナリオを実行します.
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function execute() {

        if (index >= execList.length) {

            /*
             * 終了処理の実行
             */
            __PFunc.executeFinally();

            Log.debug('['+index+'] Autorun END.');
            return;
        }

        if (isStop) {

            /*
             * 終了処理の実行
             */
            __PFunc.executeFinally();

            Log.info('Autorun STOP.');
            targetWindow.alert("自動実行を中断します。");
            return;
        }

        var execItem = execList[index];
        var execType = execItem.getAttribute("type");
        var execString = __PFunc.getExecString(execItem);

        if ("exit" == execString) {

            Log.info('Autorun Exit.');
            return;
        }

        Log.debug('['+index+'] ' + execString);
        index++;
        isNonSetTimeout = false;
        waitCount = 0;

        if (null != execType && "test" == execType) {

            Log.debug("execString:" + execString);
            /*
             * テストケースの実行
             */
            __PFunc.testExec(execString, execItem);

        } else if (null != execType && "import" == execType) {

            Log.debug("execString:" + execString);
            /*
             * 指定URLの内容をインポートする。
             */
            var importDoc;
            var url = execString + '?' + new Date().getTime();
            var xmlLS = new com.yscjp.jsapp.tools.XmlLS();
            importDoc = xmlLS.perseFromURI(url);
            if (null == importDoc) {

                var errorMsg = 'Http request error. ' +
                    xmlLS.getResultCode() + ':' +
                    xmlLS.getResultText() + '' +
                    url;
                Log.error(errorMsg);
                throw errorMsg;
            }
            var execs = importDoc.getElementsByTagName("exec");
            if (null == execs || 0 == execs.length) {

                Log.error("exec element not found.");
            }
            var replacements = execItem.getElementsByTagName("replacement");
            if (null != execs || 0 != execs.length) {

                for (var i = 0; i < execs.length; i++) {
                    var orgExecString = __PFunc.getExecString(execs[i]);
                    for (var j = 0; j < replacements.length; j++) {
                        var src = replacements[j].getAttribute("src");
                        var dest = replacements[j].getAttribute("dest");
                        orgExecString = orgExecString.replace(src, dest);
                        Log.debug("orgExecString:" + orgExecString);
                    }
                    execs[i].setAttribute("value", orgExecString);
                }
            }
            var start = index - 1;
            execList.splice(start, 1, execs[0]);
            Log.debug("splice: start:" + start + " execs.length:" + execs.length);
            for (var i = 1; i < execs.length; i++) {
                //execList.push(execs[i]);
                execList.splice(start + i, 0, execs[i]);
                Log.debug("splice: start:" + (start + i));
            }
            for (var i = 0; i < execList.length; i++) {
                Log.debug("Import:" + __PFunc.getExecString(execList[i]));
            }

            index--;

        } else if (null != execType && "branch" == execType) {

            var id = null;

            var rtn = __PFunc.executeString(execString);

            Log.debug("eval return:" + rtn);

            if (rtn) {

                id = execItem.getAttribute("then");

            } else {

                id = execItem.getAttribute("else");
            }

            __PFunc.gotoId(id);

        } else if (null != execType && "goto" == execType) {

            var id = execItem.getAttribute("destination");

            __PFunc.gotoId(id);

        } else {

            /*
             * 指定の文字列を実行する。
             */
            __PFunc.executeString(execString);
        }

        Autorun.delayTime = settingDelayTime;
        var specifiedDelayTime = execItem.getAttribute("delayTime");

        if (null != specifiedDelayTime && "" != specifiedDelayTime) {

            Log.debug('specifiedDelayTime:' + specifiedDelayTime);
            Autorun.delayTime = specifiedDelayTime - 0;
        }

        if (!isNonSetTimeout) {

            Log.debug('delayTime:' + Autorun.delayTime);
            if (isUseTargetwinTimer && targetWindow && targetWindow.opener) {

                var nextItem = execList[index];
                var nextString = __PFunc.getExecString(nextItem);
                if (-1 != nextString.search(/contentLoadWait/)) {

                    Log.debug('com.yscjp.jsapp.autorun.Autorun.execute [setTimeout]'+nextString);
                    setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);

                } else {

                    Log.debug('com.yscjp.jsapp.autorun.Autorun.execute [targetWindow.setTimeout]'+nextString);
                    targetWindow.setTimeout('opener.com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
                }

            } else {

                Log.debug('com.yscjp.jsapp.autorun.Autorun.execute [setTimeout]');
                setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
            }
        }

        isNonSetTimeout = false;
    }


    __PFunc.executeString = function (execString) {

        try {

            return  eval(execString);

        } catch (e) {

            __PFunc.executeFinally();

            throw e;
        }
    };


    __PFunc.executeFinally = function () {

        if (finallyExec) {

            /*
             * 終了処理が定義されていれば、呼び出す。
             */
            try {

                eval(finallyExec);

            } catch (e) {

                Log.error(e.name + ':' + e.message);
                if (e.fileName) {

                    Log.error(e.fileName + ':' + e.lineNumber);
                }
                if (e.fileName) {

                    Log.error(e.fileName);
                }
            }
        }
    };


    __PFunc.gotoId = function (id) {

        var newIndex = __PFunc.searchId(id);
        if (-1 == newIndex) {

            Log.debug('__PFunc.gotoId: id not found. ' + id);
            return;
        }

        index = newIndex;
        Log.debug("__PFunc.gotoId: index:" + index);
    };

    __PFunc.getExecString = function (execItem) {

        if (null == execItem) {

            return "";
        }

        var execString = execItem.getAttribute("value");
        if (!execString) {

            var firstChild = execItem.firstChild;
            if (firstChild) {

                execString = firstChild.data;

            } else {

                execString = "";
            }
        }

        return execString;
    };

    __PFunc.testExec = function (execString, execItem) {

        com.yscjp.jsapp.tools.TestRunner.run(execItem.getAttribute("name"), execString);
    };

    /**
     * 自動実行のターゲットとなる window を設定します.
     * 自動実行中にコンテンツのロードが発生する場合、必ずこのメソッドでターゲットとなる window を
     * 設定しなければいけません。
     * @param _targetWindow 繰り返すテストのインデックス
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function setTargetWindow(_targetWindow) {

        targetWindow = _targetWindow;
        newWindow = targetWindow;
        beforBody = targetWindow.document.body;

        /*
         * 最初にロードしたコンテンツに対するイベントリスナの設定
         */
        JSEvent.addEventListener(targetWindow, 'unload', __PFunc.contentUnloadListener, false);
        if ("undefined" != typeof targetWindow.onmouseout) {

            JSEvent.addEventListener(targetWindow, 'mouseout', __PFunc.mouseoutListener, false);

        } else if ("undefined" != typeof targetWindow.document.onmouseout) {

            JSEvent.addEventListener(targetWindow.document, 'mouseout', __PFunc.mouseoutListener, false);
        }
    }

    /**
     * 指定されたインデックスから、テストを指定の回数繰り返します.
     * countを指定しない場合、1回繰り返します。
     * テストの繰り返しは、入れ子に出来ません。
     * @param newIndex 繰り返すテストのインデックス
     * @param newCount 繰り返す回数
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    __PFunc.repeatFromIndex = function (newIndex, newCount) {

        if ('undefined' == typeof newCount) {

            newCount = 1;
        }

        if (-1 == repeatCount) {

            repeatCount = newCount;

        } else if (0 == repeatCount) {

            repeatCount = -1;
            return;

        } else {

            repeatCount --;
        }

        if (0 == repeatCount) {

            repeatCount = -1;
            return;
        }

        index = newIndex;
        Log.debug("repeatCount:" + repeatCount);
    };
    function repeatFromIndex(newIndex, newCount) {

        __PFunc.repeatFromIndex(newIndex, newCount);
    }

    /**
     * 指定されたインデックスから、テストを指定の回数繰り返します.
     * 旧バージョンとの互換性のためのメソッドです。
     * @param newIndex 繰り返すテストのインデックス
     * @param newCount 繰り返す回数
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function setIndex(newIndex, newCount) {

        __PFunc.repeatFromIndex(newIndex, newCount);
    }

    /**
     * 指定されたインデックスから、テストを指定の回数繰り返します.
     * countを指定しない場合、1回繰り返します。
     * テストの繰り返しは、入れ子に出来ません。
     * @param newIndex 繰り返すテストのインデックス
     * @param newCount 繰り返す回数
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    __PFunc.repeatFromId = function (id, newCount) {

        var newIndex = __PFunc.searchId(id);
        if (-1 == newIndex) {

            Log.warning("Id not find. id=" + id);
            return;
        }

        __PFunc.repeatFromIndex(newIndex, newCount);
    };
    function repeatFromId(id, newCount) {

        __PFunc.repeatFromId(id, newCount);
    }

    /**
     * 指定されたidのインデックスを返します。
     * idが存在しない場合、-1を返します。
     * @param id インデックス検索するid
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    __PFunc.searchId = function (id) {

        var newIndex = -1;
        for (var i = 0; i < execList.length; i++) {

            var exec = execList[i];
            if (id == exec.getAttribute("id")) {

                newIndex = i;
                break;
            }
        }

        return newIndex;
    };

    /**
     * ページの遷移を発生させるイベントの前に呼び出します。
     * 何もしません。
     */
    __PFunc.preContentLoad = function () {
    };
    function preContentLoad() {
        __PFunc.preContentLoad();
    }

    /**
     * コンテンツのロード完了を待ちます.
     * ページの遷移が起こるイベントの後に、必ず呼び出してください。
     */
    function contentLoadWait() {

        isNonSetTimeout = true;

        if (maxSurveillanceCount < waitCount) {

            Log.debug('Load timeout. ' + waitCount + "/" + maxSurveillanceCount);
            throw new Error('Load timeout.');
            return;
        }
        waitCount ++;

        var isLoad = false;

        /*
         * IE7以降では、documentに正しくアクセス出来無いタイミングが存在する。
         * この様なケースでは、"書き込みできません"・"アクセスが拒否されました"といった
         * パーミッション系のエラーが発生するが、実際には単なるすれ違い。
         * 回避策は、try ～ catch で例外を無視することぐらい。
         * IE6では、新しいページを読み込んでも、documentオブジェクトが変わらないという仕様なので、
         * この辺の変更が原因かも。
         */
        try {

            Log.debug('readyState:' + targetWindow.document.readyState +
                    ' href:' + targetWindow.location.href);

            if (beforBody == targetWindow.document.body) {

                Log.debug('contentLoadWait not body.');
                return;
            }

            if ('complete' != targetWindow.document.readyState) {

                Log.debug('contentLoadWait not complete.');
                return;
            }

            Log.debug('contentLoadWait complete. ' + targetWindow.location.href);
            beforBody = targetWindow.document.body;

            isLoad = true;

        } catch (e) {

            Log.warning("com.yscjp.jsapp.autorun.Autorun#contentLoadWait:" + e.name + " " + e.message);

        } finally {

            if (isLoad) {

                waitCount = 0;

                __PFunc.contentLoadListener();

                isLoaded = false;
                Log.debug('contentLoadWait_delayTime:' + Autorun.delayTime);
                if (isUseTargetwinTimer && targetWindow && targetWindow.opener) {

                    Log.debug('com.yscjp.jsapp.autorun.Autorun.execute [targetWindow.setTimeout]');
                    targetWindow.setTimeout('opener.com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);

                } else {

                    Log.debug('com.yscjp.jsapp.autorun.Autorun.execute [setTimeout]');
                    setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
                }

            } else {

                setTimeout('com.yscjp.jsapp.autorun.Autorun.contentLoadWait()', surveillanceinterval);
            }
        }
    }

    /**
     * 指定時間後に、後続の処理を続行します。
     */
    __PFunc.contentLoadDelayTimeWait = function (delayTime) {

        isNonSetTimeout = true;
        //beforHref = '';
        beforBody = null;
        setTimeout('contentLoadWait()', delayTime);
    };
    function contentLoadDelayTimeWait(delayTime) {

        __PFunc.contentLoadDelayTimeWait(delayTime);
    }

    function setNonSetTimeout() {

        isNonSetTimeout = true;
    }

    __PFunc.contentLoadListener = function () {

        if (loadListener) {

            loadListener();
        }

        JSEvent.addEventListener(targetWindow, 'beforeunload', __PFunc.contentBeforeUnloadListener, false);
        JSEvent.addEventListener(targetWindow, 'unload', __PFunc.contentUnloadListener, false);
        if ("undefined" != typeof targetWindow.onmouseout) {

            JSEvent.addEventListener(targetWindow, 'mouseout', __PFunc.mouseoutListener, false);

        } else if ("undefined" != typeof targetWindow.document.onmouseout) {

            JSEvent.addEventListener(targetWindow.document, 'mouseout', __PFunc.mouseoutListener, false);
        }
    };

    __PFunc.contentBeforeUnloadListener = function () {
    };

    __PFunc.contentUnloadListener = function () {

        if (unloadListener) {

            unloadListener();
        }

        if ("undefined" != typeof targetWindow.onmouseout) {

            JSEvent.removeEventListener(targetWindow, 'mouseout', __PFunc.mouseoutListener, false);

        } else if ("undefined" != typeof targetWindow.document.onmouseout) {

            JSEvent.removeEventListener(targetWindow.document, 'mouseout', __PFunc.mouseoutListener, false);
        }
    };

    __PFunc.mouseoutListener = function (e) {

        if (null != e.relatedTarget) {
            com.yscjp.jsapp.util.Log.debug('### Not Stop: e.target=' + e.target + " e.relatedTarget=" + e.relatedTarget);
            return;
        }

        var clientX = e.clientX;
        var clientY = e.clientY;
        if (0 < clientX && 0 < clientY) {

            var innerWidth = com.yscjp.jsapp.util.UA.getInnerWidth(targetWindow);
            var innerHeight = com.yscjp.jsapp.util.UA.getInnerHeight(targetWindow);
            if (innerWidth > clientX && innerHeight > clientY) {

                com.yscjp.jsapp.util.Log.debug("### Not Stop :clientX=" + clientX + " clientY=" + clientY +
                                               " innerWidth=" + innerWidth + " innerHeight=" + innerHeight);
                return;
            }
        }

        com.yscjp.jsapp.util.Log.debug('### Stop e.target:' + e.target);
        com.yscjp.jsapp.util.Log.debug("### Stop :clientX=" + clientX + " clientY=" + clientY +
                                       " innerWidth=" + innerWidth + " innerHeight=" + innerHeight);
        __PFunc.stop();
    };

    __PFunc.closeListener = function () {

    };

    __PFunc.stop = function () {
        if (isOutStop) {

            isStop = true;
        }
    };

    Autorun.initialize = initialize;
    Autorun.run = run;
    Autorun.setTargetWindow = setTargetWindow;
    Autorun.execute = execute;
    Autorun.setIndex = setIndex;
    Autorun.contentLoadWait = contentLoadWait;
    Autorun.setNonSetTimeout = setNonSetTimeout;

    com.yscjp.jsapp.util.StackTrace.insert(__PFunc, "com.yscjp.jsapp.autorun.Autorun.__PFunc");

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.Autorun,
                                       "com.yscjp.jsapp.autorun.Autorun");


com.yscjp.jsapp.autorun.CommandCreator = function () {};
(function() {

    var CommandCreator = com.yscjp.jsapp.autorun.CommandCreator;
    function createTextCommand(commandText) {
        return com.yscjp.jsapp.autorun.TextCommand.createCommand(commandText);
    }
    function createCommentCommand(commandText) {
        return com.yscjp.jsapp.autorun.CommentCommand.createCommand(commandText);
    }
    function createEventCommand(e) {
        return com.yscjp.jsapp.autorun.EventCommand.createCommand(e);
    }
    function createKeypressCommand(e) {
        var command = new com.yscjp.jsapp.autorun.KeypressCommand();
        command.setEvent(e);
        return command;
    }
    function createMouseMoveCommand(e) {
        return com.yscjp.jsapp.autorun.MouseMoveCommand.createCommand(e);
    }

    CommandCreator.prototype.createTextCommand = createTextCommand;
    CommandCreator.prototype.createCommentCommand = createCommentCommand;
    CommandCreator.prototype.createEventCommand = createEventCommand;
    CommandCreator.prototype.createKeypressCommand = createKeypressCommand;
    CommandCreator.prototype.createMouseMoveCommand = createMouseMoveCommand;

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.CommandCreator,
                                       "com.yscjp.jsapp.autorun.CommandCreator");


com.yscjp.jsapp.autorun.EventCapture = function () {};
(function() {

    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var WindowSurveillance = com.yscjp.jsapp.tools.WindowSurveillance;

    var commandCreator = new com.yscjp.jsapp.autorun.CommandCreator();

    var JSEvent = com.yscjp.jsapp.JSEvent;

    var Log = com.yscjp.jsapp.util.Log;
    var EventCapture = com.yscjp.jsapp.autorun.EventCapture;

    var beforBody = null;

    var eventList = new Array();

    /** trueの場合、モディファイキーのイベントもキャプチャします。 */
    EventCapture.modifyEventCapture = false;
    /** trueの場合、連続したkeyPress・keyReleaseを、keyTypeに纏めません。 */
    EventCapture.pressReleaseCapture = true;
    /** trueの場合、label要素からのclickを無視します。 */
    EventCapture.labelclick = true;

    /*
     * プライベートメソッドを管理する内部クラス。
     * トレースを取得する為に、一元管理していつ必要があるため、このような実装方法としている。
     */
    var __PFunc = function () {};

    /*
     * イベントキャプチャ中であることを示すフラグ。
     * 初期値は、falseです。
     * captureLoad()でtrueとし、captureEnd()でfalseとします。
     */
    var captureNow = false;
    var manualTestNow = false;
    var targetWindow = null;
    var surveillance = null;
    var isContentLoadWaitNeed = false;
    var onloadHdl = null;
    var unonloadHdl = null;

    /**
     * EventCapture の初期化を行います.
     * 各パラメータの値を null とした場合、直前に設定した値が有功となります。
     * @param {boolean} _modifyEventCapture モディファイキーのイベントキャプチャ有無を指定します。
     * このパラメータのデフォルトはfalseで、モディファイキー（Shift・Ctrl・Alt・Meta）のキーボードイベントは無視します。
     * trueの場合、モディファイキーのイベントもキャプチャします。
     * @param {boolean} _pressReleaseCapture 連続したkeyPress・keyReleaseを、keyTypeに纏めるかを指定します。
     * このパラメータのデフォルトはtrueで、keyPressとkeyReleaseのペアではなく、keyType１つをイベントとして登録します。
     * trueの場合、連続したkeyPress・keyReleaseを、keyTypeに纏めません。
     * @param {boolean} _labelclick label要素からのclickを無視するかを指定します。
     * label要素からのイベントを処理してしまうと、Webブラウザによっては同じイベントが複数回発生することになるので、
     * これを回避するためのものです。デフォルトはtrueです。
     * trueの場合、label要素からのclickを無視します。
     * @memberOf com.yscjp.jsapp.autorun.Autorun
     */
    function initialize (_modifyEventCapture,
                         _pressReleaseCapture,
                         _labelclick,
                         _onloadHdl,
                         _unonloadHdl) {

        EventCapture.modifyEventCapture = ArgChk.notNull(_modifyEventCapture, EventCapture.modifyEventCapture);
        EventCapture.pressReleaseCapture = ArgChk.notNull(_pressReleaseCapture, EventCapture.pressReleaseCapture);
        EventCapture.labelclick = ArgChk.notNull(_labelclick, EventCapture.labelclick);

        onloadHdl = ArgChk.notNull(_onloadHdl, null);
        unonloadHdl = ArgChk.notNull(_unonloadHdl, null);

        eventList.length = 0;

    };

    function setTargetWindow(_targetWindow) {
        targetWindow = _targetWindow;
    }

    /**
     *
     */
    function captureInit() {

        var loadURL = document.getElementById('targeturl').value;
        var command = commandCreator.createTextCommand('loadContent(\'' + loadURL + '\')');
        eventList.push(command);
        captureNow = true;
        surveillance = new WindowSurveillance(targetWindow, 10, __PFunc.loadListener, null/*, targetWindow*/);
        isContentLoadWaitNeed = false;
        surveillance.start();
    }

    __PFunc.captureLoadInit = function () {

        JSEvent.addEventListener(targetWindow, 'unload', __PFunc.unloadListener, false);
        JSEvent.addEventListener(targetWindow.document, 'mousedown', __PFunc.mousedownListener, false);
        JSEvent.addEventListener(targetWindow.document, 'mouseup', __PFunc.mouseupListener, false);
        JSEvent.addEventListener(targetWindow.document, 'click', __PFunc.mouseclickListener, false);
        JSEvent.addEventListener(targetWindow.document, 'dblclick', __PFunc.mousedblclickListener, false);
        JSEvent.addEventListener(targetWindow.document, 'keydown', __PFunc.keydownListener, false);
        JSEvent.addEventListener(targetWindow.document, 'keyup', __PFunc.keyupListener, false);
        //JSEvent.addEventListener(targetWindow, 'keypress', keypressListener, false);
    };

    __PFunc.loadListener = function (e) {

        if (!captureNow) {

            return;
        }

        var command = commandCreator.createTextCommand('contentLoadWait()');
        beforBody = targetWindow.document.body;

        Log.debug('COMMAND:' + command.toCommandString());
        if (isContentLoadWaitNeed) {

            eventList.push(command);

        } else {

            /*
             * 初回は、すでにコンテンツのロードを確認しているので、'contentLoadWait()' は不要。
             * 次回から必要となる。
             */
            isContentLoadWaitNeed = true;
        }

        var title = targetWindow.document.title;
        if (!title) {

            /* TODO XMLのコメントとして使用できない文字をエンコードする必要が有る。 */
            title = targetWindow.location.href.replace("--", "&#45;&#45;");
        }
        var comment = commandCreator.createCommentCommand(title);
        eventList.push(comment);

        __PFunc.captureLoadInit();

        if (onloadHdl) {

            onloadHdl();
        }
    };

    __PFunc.unloadListener = function (e) {

        if (!captureNow) {

            return;
        }

        if (unonloadHdl) {

            unonloadHdl();
        }
    };

    function manualTestInit() {

        manualTestNow = true;
        surveillance = new WindowSurveillance(targetWindow, 10, __PFunc.manualTestLoadListener, null);
        surveillance.start();
    }

    __PFunc.manualTestLoadInit = function () {

//        JSEvent.addEventListener(targetWindow, 'unload', __PFunc.manualTestUnloadListener, false);
        JSEvent.addEventListener(targetWindow, 'beforeunload', __PFunc.manualTestUnloadListener, false);
    };

    __PFunc.manualTestLoadListener = function (e) {

        if (!manualTestNow) {

            return;
        }

        __PFunc.manualTestLoadInit();

        if (onloadHdl) {

            onloadHdl();
        }
    };

    __PFunc.manualTestUnloadListener = function (e) {

        if (!manualTestNow) {

            return;
        }

        if (unonloadHdl) {

            unonloadHdl();
        }
    };

    function addOnLoad() {

        if (onloadHdl) {

            JSEvent.addEventListener(targetWindow, 'load', onloadHdl, false);
        }
    }

    __PFunc.mousedownListener = function (e) {

        if (!captureNow) {

            return;
        }

        e = JSEvent.createEvent(e);

        /*
         * マウスダウンの場合は、無条件でターゲット要素までマウスカーソルを移動させる。
         */
        var command = commandCreator.createMouseMoveCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);

        command = commandCreator.createEventCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };

    __PFunc.mouseupListener = function (e) {

        if (!captureNow) {

            return;
        }

        /*
         * 直前のイベントと発生位置が一定以上離れている場合、マウスカーソルを移動させる。
         */
        if (1 <= eventList.length) {

            var lastEvent = eventList[eventList.length - 1];
            if ('mousedown' == lastEvent.getType()) {

                var threshold = 10;

                var diffX = Math.abs(e.clientX - lastEvent.e.clientX);
                var diffY = Math.abs(e.clientY - lastEvent.e.clientY);
                if (diffX > threshold || diffY > threshold) {

                    var command = commandCreator.createMouseMoveCommand(e);
                    Log.debug('COMMAND:' + command.toCommandString());
                    eventList.push(command);
                }
            }
        }
        e = JSEvent.createEvent(e);
        var command = commandCreator.createEventCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };

    __PFunc.mouseclickListener = function (e) {

        if (!captureNow) {

            return;
        }

        e = JSEvent.createEvent(e);

        if ("LABEL" == e.target.tagName && EventCapture.labelclick) {

            return;
        }

        /*
         * マウスクリックイベントの場合、直前のイベントはマウスアップ・マウスダウンの筈。
         * これらを削除して、マウスクリックイベントを追加する。
         * 直前のイベントがマウスアップ・マウスダウンではない場合、削除をしないで追加だけ行う。
         */
        Log.debug('eventList.length:' + eventList.length);
        if (2 <= eventList.length) {

            var lastEvent = eventList[eventList.length - 1];
            var towPrevEvent = eventList[eventList.length - 2];

            Log.debug('lastEvent:' + lastEvent.toCommandString() +
                      '  towPrevEvent:' + towPrevEvent.toCommandString());

            if ('mouseup' == lastEvent.getType() && 'mousedown' == towPrevEvent.getType()) {

                Log.debug('Delete Command:' + eventList.pop().toCommandString());
                Log.debug('Delete Command:' + eventList.pop().toCommandString());
            }
        }

        var command = commandCreator.createEventCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };

    __PFunc.mousedblclickListener = function (e) {

        if (!captureNow) {

            return;
        }

        e = JSEvent.createEvent(e);

        /*
         * マウスダブルクリックイベントの場合、直前のイベントはマウスクリックが連続して2つの筈。
         * これらを削除して、マウスダブルクリックイベントを追加する。
         * 直前のイベントがマウスクリック2つではない場合、削除をしないで追加だけ行う。
         */
        if (2 <= eventList.length) {

            var lastEvent = eventList[eventList.length - 1];
            var towPrevEvent = eventList[eventList.length - 2];

            if ('click' == lastEvent.getType() && 'click' == towPrevEvent.getType()) {

                eventList.pop();
                eventList.pop();
            }
        }

        var command = commandCreator.createEventCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };

    __PFunc.keydownListener = function (e) {

        if (!captureNow) {

            return;
        }

        e = JSEvent.createEvent(e);

        if (!EventCapture.pressReleaseCapture) {

            return;
        }

        if (!EventCapture.modifyEventCapture) {

            /*
             * モディファイキーの場合キャプチャしない
             */
            if (e.keyCode == 16 ||  // Shift
                e.keyCode == 17 ||  // Ctrl
                e.keyCode == 18 ||  // Alt
                e.keyCode == 224) { // Meta

                return;
            }
        }
        var command = commandCreator.createEventCommand(e);
        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };

    __PFunc.keyupListener = function (e) {

        if (!captureNow) {

            return;
        }

        e = JSEvent.createEvent(e);

        if (!EventCapture.modifyEventCapture) {

            /*
             * モディファイキーの場合キャプチャしない
             */
            if (e.keyCode == 16 ||  // Shift
                e.keyCode == 17 ||  // Ctrl
                e.keyCode == 18 ||  // Alt
                e.keyCode == 224) { // Meta

                Log.trace('---------- modifyEventCapture(' + e.keyCode + ') ----------');
                return;
            }
        }

        var command;

        if (!EventCapture.pressReleaseCapture) {

            /*
             * keyTypeに変換する。
             */
            command = commandCreator.createKeypressCommand(e);

        } else {

            command = commandCreator.createEventCommand(e);
        }

        Log.debug('COMMAND:' + command.toCommandString());
        eventList.push(command);
    };


    function captureEnd() {

        /*
         * キャプチャを終了します。
         */
        captureNow = false;

        var capturetext = "";
        capturetext = "<doc>\n";

        for (var i = 0; i < eventList.length; i++) {

            var command = eventList[i];

            if ("CommentCommand" == command.getCommandType()) {

                /* コメント */
                capturetext += '<!-- ' + command.toCommandString() + ' -->\n';
                continue;
            }

            if ("KeypressCommand" == command.getCommandType() &&
                command.isPrintableCharacters()) {

                /*
                 * KeypressCommand の場合は、直後のコマンドも KeypressCommand であれば、
                 * 間隔時間を短く設定する。
                 */
                var nextCommand = eventList[i + 1];
                if (nextCommand &&
                    "KeypressCommand" == nextCommand.getCommandType() &&
                    nextCommand.isPrintableCharacters()) {

                    if (document.getElementById("sequencekeytype").checked) {

                        /*  */
                        capturetext += '<exec delayTime="50"><![CDATA[' + command.toCommandString() + ']]></exec>\n';
                        continue;
                    }
                }
            }

            //Log.debug('command:' + command);
            capturetext += '<exec><![CDATA[' + command.toCommandString() + ']]></exec>\n';
        }

        capturetext += '</doc>';

        return capturetext;
    }

    com.yscjp.jsapp.util.StackTrace.insert(__PFunc, "com.yscjp.jsapp.autorun.EventCapture.__PFunc");

    EventCapture.captureInit = captureInit;
    EventCapture.manualTestInit = manualTestInit;
    EventCapture.initialize = initialize;
    EventCapture.setTargetWindow = setTargetWindow;
    EventCapture.captureEnd = captureEnd;
    EventCapture.addOnLoad = addOnLoad;

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.EventCapture,
                                       "com.yscjp.jsapp.autorun.EventCapture");


com.yscjp.jsapp.autorun.AbstractTextCommand = function () {};
(function() {

    var AbstractTextCommand = com.yscjp.jsapp.autorun.AbstractTextCommand;

    AbstractTextCommand.prototype.getCommandType = function () {

        return "AbstractTextCommand";
    };

    AbstractTextCommand.prototype.getType = function () {

        return "";
    };

    AbstractTextCommand.prototype.setCommandText = function (commandText) {

        this.commandText = commandText;
    };

    AbstractTextCommand.prototype.toCommandString = function () {

        return this.commandText;
    };

})();

com.yscjp.jsapp.autorun.TextCommand = function () {};
(function() {

    var TextCommand = com.yscjp.jsapp.autorun.TextCommand;

    TextCommand.prototype = new com.yscjp.jsapp.autorun.AbstractTextCommand();

    TextCommand.createCommand = function (commandText) {

        var command = new TextCommand();
        command.setCommandText(commandText);
        return command;
    };

    TextCommand.prototype.getCommandType = function () {

        return "TextCommand";
    };

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.TextCommand,
                                       "com.yscjp.jsapp.autorun.TextCommand");


com.yscjp.jsapp.autorun.CommentCommand = function () {};
(function() {

    var CommentCommand = com.yscjp.jsapp.autorun.CommentCommand;

    CommentCommand.prototype = new com.yscjp.jsapp.autorun.AbstractTextCommand();

    CommentCommand.createCommand = function (commandText) {

        var command = new CommentCommand();
        command.setCommandText(commandText);
        return command;
    };

    CommentCommand.prototype.getCommandType = function () {

        return "CommentCommand";
    };

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.CommentCommand,
                                       "com.yscjp.jsapp.autorun.CommentCommand");


com.yscjp.jsapp.autorun.EventCommand = function () {};
(function() {

    var EventCommand = com.yscjp.jsapp.autorun.EventCommand;

    EventCommand.createCommand = function (e) {

        var command = null;

        switch (e.type) {
        case "keydown"  :

            command = new com.yscjp.jsapp.autorun.KeydownCommand();
            break;

        case "keyup"    :

            command = new com.yscjp.jsapp.autorun.KeyupCommand();
            break;

        case "keypress" :

            command = new com.yscjp.jsapp.autorun.KeypressCommand();
            break;

        case "mousedown" :

            command = new com.yscjp.jsapp.autorun.MousedownCommand();
            break;

        case "mouseup"   :

            command = new com.yscjp.jsapp.autorun.MouseupCommand();
            break;

        case "click"     :

            command = new com.yscjp.jsapp.autorun.MouseclickCommand();
            break;

        case "dblclick"  :

            command = new com.yscjp.jsapp.autorun.MouseDblclickCommand();
            break;

        }

        command.setEvent(e);
        return command;
    };

    EventCommand.prototype.getCommandType = function () {

        return "EventCommand";
    };

    EventCommand.prototype.setEvent = function (e) {

        this.e = e;
    };

    EventCommand.prototype.getType = function () {

        return this.e.type;
    };

})();

com.yscjp.jsapp.autorun.KeyCommand = function () {};
(function() {

    var KeyCommand = com.yscjp.jsapp.autorun.KeyCommand;

    KeyCommand.prototype = new com.yscjp.jsapp.autorun.EventCommand();

    KeyCommand.prototype.getCommandType = function () {

        return "KeyCommand";
    };

    KeyCommand.prototype.isPrintableCharacters = function () {

        if (0x20 <= this.e.keyCode && this.e.keyCode <= 0x7e) {

            return true;
        }

        return false;
    };

    KeyCommand.convertCode = function (jsCode) {

//        if (13 == jsCode) {
//
//            return 10;
//        }

        return jsCode;
    };

})();

com.yscjp.jsapp.autorun.KeydownCommand = function () {};
(function() {

    var KeyCommand = com.yscjp.jsapp.autorun.KeyCommand;
    var KeydownCommand = com.yscjp.jsapp.autorun.KeydownCommand;

    KeydownCommand.prototype = new com.yscjp.jsapp.autorun.KeyCommand();

    KeydownCommand.prototype.getCommandType = function () {

        return "KeydownCommand";
    };

    KeydownCommand.prototype.toCommandString = function () {

        return 'jsRobot.keyPress(' + KeyCommand.convertCode(this.e.keyCode) + ', ' +
                                     this.e.ctrlKey + ',' +
                                     this.e.shiftKey + ',' +
                                     this.e.altKey + ',' +
                                     this.e.metaKey +
                                     ')';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.KeydownCommand,
                                       "com.yscjp.jsapp.autorun.KeydownCommand");


com.yscjp.jsapp.autorun.KeyupCommand = function () {};
(function() {

    var KeyCommand = com.yscjp.jsapp.autorun.KeyCommand;
    var KeyupCommand = com.yscjp.jsapp.autorun.KeyupCommand;

    KeyupCommand.prototype = new com.yscjp.jsapp.autorun.KeyCommand();

    KeyupCommand.prototype.getCommandType = function () {

        return "KeyupCommand";
    };

    KeyupCommand.prototype.toCommandString = function () {

        return 'jsRobot.keyRelease(' + KeyCommand.convertCode(this.e.keyCode) + ', ' +
                                       this.e.ctrlKey + ',' +
                                       this.e.shiftKey + ',' +
                                       this.e.altKey + ',' +
                                       this.e.metaKey +
                                       ')';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.KeyupCommand,
                                       "com.yscjp.jsapp.autorun.KeyupCommand");


com.yscjp.jsapp.autorun.KeypressCommand = function () {};
(function() {

    var KeyCommand = com.yscjp.jsapp.autorun.KeyCommand;
    var KeypressCommand = com.yscjp.jsapp.autorun.KeypressCommand;

    KeypressCommand.prototype = new com.yscjp.jsapp.autorun.KeyCommand();

    KeypressCommand.prototype.getCommandType = function () {

        return "KeypressCommand";
    };

    KeypressCommand.prototype.toCommandString = function () {

        return 'jsRobot.keyType(' + KeyCommand.convertCode(this.e.keyCode) + ', ' +
                                    this.e.ctrlKey + ',' +
                                    this.e.shiftKey + ',' +
                                    this.e.altKey + ',' +
                                    this.e.metaKey +
                                    ')';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.KeypressCommand,
                                       "com.yscjp.jsapp.autorun.KeypressCommand");


com.yscjp.jsapp.autorun.MouseCommand = function () {};
(function() {

    var MouseCommand = com.yscjp.jsapp.autorun.MouseCommand;

    MouseCommand.prototype = new com.yscjp.jsapp.autorun.EventCommand();

    MouseCommand.prototype.getCommandType = function () {

        return "MouseCommand";
    };

    MouseCommand.prototype.getButtonString = function () {

        if (0 == this.e.button) {

            /*
             * 左ボタン押下
             */
            return 'Left';

        } else if (1 == this.e.button) {

            /*
             * 中ボタン押下
             */
            return  'Center';

        } else {

            /*
             * 右ボタン押下
             */
            return 'Right';

        }
    };

})();

com.yscjp.jsapp.autorun.MousedownCommand = function () {};
(function() {

    var MousedownCommand = com.yscjp.jsapp.autorun.MousedownCommand;

    MousedownCommand.prototype = new com.yscjp.jsapp.autorun.MouseCommand();

    MousedownCommand.prototype.getCommandType = function () {

        return "MousedownCommand";
    };

    MousedownCommand.prototype.toCommandString = function () {

        return 'jsRobot.mouse' + this.getButtonString() + 'Press()';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.MousedownCommand,
                                       "com.yscjp.jsapp.autorun.MousedownCommand");


com.yscjp.jsapp.autorun.MouseupCommand = function () {};
(function() {

    var MouseupCommand = com.yscjp.jsapp.autorun.MouseupCommand;

    MouseupCommand.prototype = new com.yscjp.jsapp.autorun.MouseCommand();

    MouseupCommand.prototype.getCommandType = function () {

        return "MouseupCommand";
    };

    MouseupCommand.prototype.toCommandString = function () {

        return 'jsRobot.mouse' + this.getButtonString() + 'Release()';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.MouseupCommand,
                                       "com.yscjp.jsapp.autorun.MouseupCommand");


com.yscjp.jsapp.autorun.MouseclickCommand = function () {};
(function() {

    var MouseclickCommand = com.yscjp.jsapp.autorun.MouseclickCommand;

    MouseclickCommand.prototype = new com.yscjp.jsapp.autorun.MouseCommand();

    MouseclickCommand.prototype.getCommandType = function () {

        return "MouseclickCommand";
    };

    MouseclickCommand.prototype.toCommandString = function () {

        return 'jsRobot.mouse' + this.getButtonString() + 'Click()';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.MouseclickCommand,
                                       "com.yscjp.jsapp.autorun.MouseclickCommand");


com.yscjp.jsapp.autorun.MouseDblclickCommand = function () {};
(function() {

    var MouseDblclickCommand = com.yscjp.jsapp.autorun.MouseDblclickCommand;

    MouseDblclickCommand.prototype = new com.yscjp.jsapp.autorun.MouseCommand();

    MouseDblclickCommand.prototype.getCommandType = function () {

        return "MouseDblclickCommand";
    };

    MouseDblclickCommand.prototype.toCommandString = function () {

        return 'jsRobot.mouse' + this.getButtonString() + 'Dblclick()';
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.MouseDblclickCommand,
                                       "com.yscjp.jsapp.autorun.MouseDblclickCommand");


com.yscjp.jsapp.autorun.MouseMoveCommand = function () {};
(function() {

    var MouseMoveCommand = com.yscjp.jsapp.autorun.MouseMoveCommand;
    var Log = com.yscjp.jsapp.util.Log;


    MouseMoveCommand.createCommand = function (e) {

        Log.trace("---------- MouseMoveCommand.createCommand ----------");
        var command = new MouseMoveCommand();
        command.setEvent(e);

        return command;
    };

    MouseMoveCommand.prototype = new com.yscjp.jsapp.autorun.MouseCommand();

    MouseMoveCommand.prototype.setEvent = function (e) {

        Log.trace("---------- MouseMoveCommand.prototype.setEvent ----------");
        this.e = e;
        this.targetId = e.target.id;
        this.targetName = e.target.name;
        this.targetTagName = e.target.tagName;
        this.elementIndex = -1;

        /*
         * マウスカーソルの直下に有った要素の確定
         */
        var elementList = e.target.ownerDocument.getElementsByTagName(this.targetTagName);
        for (var i = 0; i < elementList.length; i ++) {

            if (e.target == elementList.item(i)) {

                this.elementIndex = i;
                break;
            }
        }
    };

    MouseMoveCommand.prototype.getCommandType = function () {

        return "MouseMoveCommand";
    };

    MouseMoveCommand.prototype.toCommandString = function () {

        var commandString;

        if (this.targetId && 0 < this.targetId.length) {

            commandString = 'jsRobot.mouseMoveToId(\'' + this.targetId + '\')';

        } else if (this.targetName && 0 < this.targetName.length) {

            commandString = 'jsRobot.mouseMoveToName(\'' + this.targetName + '\')';

        } else {

            if (-1 == this.elementIndex) {

                Log.debug('Tag not found:' + this.targetTagName);
                return;
            }

            commandString = 'jsRobot.mouseMoveToTagNameAt(\'' + this.targetTagName + '\', ' + this.elementIndex + ')';
        }

        return commandString;
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.autorun.MouseMoveCommand,
                                       "com.yscjp.jsapp.autorun.MouseMoveCommand");
