package com.yscjp.webtools.robot;

import java.applet.Applet;
import java.awt.Color;
import java.awt.Container;
import java.awt.Frame;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.Window;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.imageio.ImageIO;


/**
 *
 * @author bison
 *
 */
public class RobotApplet extends Applet {
    public RobotApplet() {
    }

    /** デフォルトシリアルバージョンID */
    private static final long serialVersionUID = 1L;
    private Robot robot;
    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy.MM.dd_HH-mm-ss.SSS");
    private CaptureThread captureThread;
    private CaptureSizeWindow window = null;

    private volatile boolean isCaptureThreadWait = true;
    private volatile boolean isScriptThreadWait = true;

    private boolean isVerboseMessage = false;

    private Timer mouseReleaseTimer = null;

    private static Map<Integer, Integer> convertKeycodeMap = new HashMap<Integer, Integer>();
    static {

        /*
         * Javascript と Java のキーコードの違いを変換する。
         */
        //                    Javascript    Java
        convertKeycodeMap.put(   220,         92);  // VK_BACK_SLASH     220     92
        convertKeycodeMap.put(   226,         92);  // VK_BACK_SLASH     220     92
        convertKeycodeMap.put(   221,         93);  // VK_CLOSE_BRACKET  221     93
        convertKeycodeMap.put(   222,        514);  // VK_CIRCUMFLEX  222     514
        convertKeycodeMap.put(   186,        513);  // VK_COLON          186    513
        convertKeycodeMap.put(   187,         59);  // VK_SEMICOLON          187     59
        convertKeycodeMap.put(   188,         44);  // VK_COMMA          188     44
        convertKeycodeMap.put(   189,         45);  // VK_MINUS          189     45
//        convertKeycodeMap.put(    93,        525);  // VK_CONTEXT_MENU    93    525
//        convertKeycodeMap.put(    46,        127);  // VK_DELETE          46    127
        convertKeycodeMap.put(    13,         10);  // VK_ENTER           14     10
//        convertKeycodeMap.put(   124,      61440);  // VK_F13            124  61440
//        convertKeycodeMap.put(   125,      61441);  // VK_F14            125  61441
//        convertKeycodeMap.put(   126,      61442);  // VK_F15            126  61442
//        convertKeycodeMap.put(   127,      61443);  // VK_F16            127  61443
//        convertKeycodeMap.put(   128,      61444);  // VK_F17            128  61444
//        convertKeycodeMap.put(   129,      61445);  // VK_F18            129  61445
//        convertKeycodeMap.put(   130,      61446);  // VK_F19            130  61446
//        convertKeycodeMap.put(   131,      61447);  // VK_F20            131  61447
//        convertKeycodeMap.put(   132,      61448);  // VK_F21            132  61448
//        convertKeycodeMap.put(   133,      61449);  // VK_F22            133  61449
//        convertKeycodeMap.put(   134,      61450);  // VK_F23            134  61450
//        convertKeycodeMap.put(   135,      61451);  // VK_F24            135  61451
//        convertKeycodeMap.put(     6,        156);  // VK_HELP             6    156
//        convertKeycodeMap.put(    45,        155);  // VK_INSERT          45    155
//        convertKeycodeMap.put(   224,        157);  // VK_META           224    157
        convertKeycodeMap.put(   219,         91);  // VK_OPEN_BRACKET   219     91
        convertKeycodeMap.put(   190,         46);  // VK_PERIOD         190     46
        convertKeycodeMap.put(    44,        154);  // VK_PRINTSCREEN     44    154
        convertKeycodeMap.put(   191,         47);  // VK_SLASH          191     47
        convertKeycodeMap.put(   192,        512);  // VK_AT (@)          192     512
    }

    private static class CaptureThread extends Thread {

        private RobotApplet applet;
        private int x;
        private int y;
        private int width;
        private int height;
        private String name;
        private Thread jsThread = null;
//        private boolean loop = true;

        private CaptureThread(RobotApplet applet) {

            this.applet = applet;
        }

        private void setValues(int x, int y, int width, int height, String name, Thread jsThread) {

            this.x = x;
            this.y = y;
            this.width = width;
            this.height = height;
            this.name = name;
            this.jsThread = jsThread;
        }

        @Override
        public void run() {

            applet.putMessage("---------- CaptureThread start ----------");

            while (!this.isInterrupted()) {

                applet.setBackground(Color.green);

                try {

                    synchronized (this) {

                        try {

                            if (applet.isCaptureThreadWait) {

                                applet.putMessage("---------- CaptureThread wait ----------");
                                wait();
                                applet.putMessage("---------- CaptureThread wake up ----------");
                            }
                            applet.isCaptureThreadWait = true;
                            applet.execCapture(x, y, width, height, name);

                        } catch (Exception e) {

                            e.printStackTrace();
                        }
                    }

                } finally {

                    synchronized (applet) {

                        if (null != jsThread) {

                            applet.putMessage("---------- applet.notify() ----------");
                            applet.isScriptThreadWait = false;
                            applet.notify();
                        }
                        jsThread = null;
                    }
                }
            }

            applet.putMessage("---------- CaptureThread end ----------");
            applet.setBackground(Color.yellow);
        }
    }

    @Override
    public void destroy() {

        putMessage("---------- destroy ----------");
//        loop = false;
    }

    @Override
    public void init() {

        System.out.println("---------- init 0109 ----------");

        setBackground(Color.red);

        try {

            robot = new Robot();
            setBackground(Color.yellow);
//            captureThread = new CaptureThread(this);
//            loop = true;
//            captureThread.start();

        } catch (Exception e) {

            e.printStackTrace();
        }
    }

    @Override
    public void start() {

        putMessage("---------- start ----------");
        captureThread = new CaptureThread(this);
        captureThread.start();
    }

    @Override
    public void stop() {

        putMessage("---------- stop ----------");

        if (null != captureThread) {

            captureThread.interrupt();
            try {

                captureThread.join();

            } catch (Exception e) {

                e.printStackTrace();
            }
        }
    }

    public void startVerboseMessage() {

        this.isVerboseMessage = true;
    }

    public void stopVerboseMessage() {

        this.isVerboseMessage = false;
    }

    public void verboseMessage(boolean isVerboseMessage) {

//        this.isVerboseMessage = isVerboseMessage;
    }

    private void putMessage(String message) {

        if (isVerboseMessage) {

            System.out.println(message);
        }
    }

    public Point getPoint() {

        return getLocationOnScreen();
    }

    public void delay(int ms) {

        robot.delay(ms);
    }

    /**
     * 指定されたキーを押します。
     *
     * @param jsKeycode
     */
    public void keyPress(int jsKeycode) {

        keyPress(jsKeycode, false, false, false, false);
    }

    /**
     * 指定されたキーを押します。
     *
     * @param jsKeycode
     */
    public void keyPress(int jsKeycode, boolean isCtrlKey, boolean isShiftKey, boolean isAltKey, boolean isMetaKey) {

        int keycode = jsKeycode;

        if (convertKeycodeMap.containsKey(jsKeycode)) {

            keycode = convertKeycodeMap.get(jsKeycode);
        }

        putMessage("keyPress:" + keycode + " " + modifyKeyPress(isCtrlKey, isShiftKey, isAltKey, isMetaKey));

        try {

            robot.keyPress(keycode);

        } catch (Exception e) {

            System.out.println("keycode:" + keycode);
            e.printStackTrace(System.out);

        } finally {

            modifyKeyRelease(isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }
    }

    /**
     * 指定されたキーを離します。
     *
     * @param jsKeycode
     */
    public void keyRelease(int jsKeycode) {

        keyRelease(jsKeycode, false, false, false, false);
    }

    /**
     * 指定されたキーを離します。
     *
     * @param jsKeycode
     */
    public void keyRelease(int jsKeycode, boolean isCtrlKey, boolean isShiftKey, boolean isAltKey, boolean isMetaKey) {

        int keycode = jsKeycode;

        if (convertKeycodeMap.containsKey(jsKeycode)) {

            keycode = convertKeycodeMap.get(jsKeycode);
        }

        putMessage("keyRelease:" + keycode + " " + modifyKeyPress(isCtrlKey, isShiftKey, isAltKey, isMetaKey));

        try {

            robot.keyRelease(keycode);

        } catch (Exception e) {

            e.printStackTrace();

        } finally {

            modifyKeyRelease(isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }
    }

    private String modifyKeyPress(boolean isCtrlKey, boolean isShiftKey, boolean isAltKey, boolean isMetaKey) {

        if (isCtrlKey) {

            robot.keyPress(KeyEvent.VK_CONTROL);
        }

        if (isShiftKey) {

            robot.keyPress(KeyEvent.VK_SHIFT);
        }

        if (isAltKey) {

            robot.keyPress(KeyEvent.VK_ALT);
        }

        if (isMetaKey) {

            robot.keyPress(KeyEvent.VK_META);
        }

        if (isVerboseMessage) {

            return "isCtrlKey:" + isCtrlKey + " isShiftKey:" + isShiftKey + " isAltKey:" + isAltKey + " isMetaKey:" + isMetaKey;

        } else {

            return "";
        }
    }

    private String modifyKeyRelease(boolean isCtrlKey, boolean isShiftKey, boolean isAltKey, boolean isMetaKey) {

        if (isCtrlKey) {

            robot.keyRelease(KeyEvent.VK_CONTROL);
        }

        if (isShiftKey) {

            robot.keyRelease(KeyEvent.VK_SHIFT);
        }

        if (isAltKey) {

            robot.keyRelease(KeyEvent.VK_ALT);
        }

        if (isMetaKey) {

            robot.keyRelease(KeyEvent.VK_META);
        }

        if (isVerboseMessage) {

            return "isCtrlKey:" + isCtrlKey + " isShiftKey:" + isShiftKey + " isAltKey:" + isAltKey + " isMetaKey:" + isMetaKey;

        } else {

            return "";
        }
    }

    /**
     * 指定されたキーを押して離します。
     *
     * @param jsKeycode
     */
    public void keyType(int jsKeycode) {

        robot.keyPress(jsKeycode);
        robot.keyRelease(jsKeycode);
    }

    /**
     * 指定されたキャラクタのキーを押して離します。
     *
     * @param keycode
     */
    public void typeCharKey(String str) {

        char c = str.charAt(0);

        try {

            KeyChar keyChar = getKeyCode(c);
            if (null == keyChar) {

                return;
            }

            if (keyChar.isShift()) {

                keyPress(KeyEvent.VK_SHIFT);
            }

            keyType(keyChar.getKeyCode());
            //keyPress(keyChar.getKeyCode());
            //keyRelease(keyChar.getKeyCode());

            if (keyChar.isShift()) {

                keyRelease(KeyEvent.VK_SHIFT);
            }

        }
        catch (Exception e) {

            e.printStackTrace();
        }
    }

    private KeyChar getKeyCode(char c) {

        return KeyChar.getKeyChar(c);
    }

    public void inputText(String s) {

        putMessage("s[" + s + "]");

        for (int i = 0; i < s.length(); i++) {

            typeCharKey(s.substring(i, i + 1));
        }
    }

    /**
     * 指定したスクリーン座標にマウスポインタを移動します。
     *
     * @param keycode
     */
    public void mouseMove(int x, int y) {

        try {

            robot.mouseMove(x, y);

        } catch (Exception e) {

            e.printStackTrace();
        }
    }

    /**
     * ホイールマウスのホイールスクロールを回転させます。
     *
     * @param keycode
     */
    public void mouseWheel(int wheelAmt) {

        robot.mouseWheel(wheelAmt);
    }

    /**
     * 1 つまたは複数のマウスボタンを押します。
     *
     * @param keycode
     */
    public void mousePress(final int buttons) {

        try {

            robot.mousePress(buttons);

        } finally {

            final RobotApplet __this__ = this;

            /*
             * タイマの設定
             * 一定時間マウスキーがリリースされない場合、リリースする。
             */
            mouseReleaseTimer = new Timer();
            mouseReleaseTimer.schedule(new TimerTask() {

                @Override
                public void run() {

                    __this__.mouseRelease(buttons);
                    System.out.println("---------- Mouse released ----------");
                }
            }, 30 * 1000);
        }
    }

    /**
     * 1 つまたは複数のマウスボタンを離します。
     *
     * @param keycode
     */
    public void mouseRelease(int buttons) {

        if (null != mouseReleaseTimer) {

            mouseReleaseTimer.cancel();
            mouseReleaseTimer = null;
        }

        robot.mouseRelease(buttons);
    }

    /**
     * 1 つまたは複数のマウスボタンをクリックします。
     * @param buttons
     */
    public void mouseClick(int buttons) {
        mousePress(buttons);
        mouseRelease(buttons);
    }

    /**
     * 1 つまたは複数のマウスボタンを押します。
     *
     * @param keycode
     */
    public void mousePress(boolean leftButton, boolean centerButton, boolean rightButton) {

        mousePress(getButton(leftButton, centerButton, rightButton));
    }

    /**
     * 1 つまたは複数のマウスボタンを離します。
     *
     * @param keycode
     */
    public void mouseRelease(boolean leftButton, boolean centerButton, boolean rightButton) {

        mouseRelease(getButton(leftButton, centerButton, rightButton));
    }

    /**
     * 1 つまたは複数のマウスボタンをクリックします。
     * @param buttons
     */
    public void mouseClick(boolean leftButton, boolean centerButton, boolean rightButton) {

        int buttons = getButton(leftButton, centerButton, rightButton);
        mousePress(buttons);
        mouseRelease(buttons);
    }

    public void mouseLeftClick() {

        mouseClick(true, false, false);
    }

    public void mouseCenterClick() {

        mouseClick(false, true, false);
    }

    public void mouseRightClick() {

        mouseClick(false, false, true);
    }

    private int getButton(boolean leftButton, boolean centerButton, boolean rightButton) {

        int buttons = 0;
        if (leftButton) {

            buttons = buttons | InputEvent.BUTTON1_MASK;
        }
        if (centerButton) {

            buttons = buttons | InputEvent.BUTTON2_MASK;
        }
        if (rightButton) {

            buttons = buttons | InputEvent.BUTTON3_MASK;
        }

        return buttons;
    }

    public void capture(int x, int y, int width, int height, String name) {

        synchronized (RobotApplet.class) {

            putMessage(">>>>> [START_capture]\t" + name);
            isScriptThreadWait = true;

            try {

                captureThread.setValues(x, y, width, height, name, Thread.currentThread());

            } finally {

                synchronized (captureThread) {

                    putMessage("---------- captureThread.notify ----------");
                    isCaptureThreadWait = false;
                    captureThread.notify();
                }
            }

            synchronized (this) {

                try {

                    if (isScriptThreadWait) {

                        putMessage("---------- capture wait ----------");
                        wait(30000);
                    }
                    putMessage("---------- capture wait end ----------");

                } catch (Exception e) {

                    e.printStackTrace();
                }
            }
            putMessage("<<<<< [END_capture]\t" + name);

        }
    }

    private void execCapture(int x, int y, int width, int height, String name) {

        putMessage(">>>>>>> [START_execCapture]\t" + name);

        try {

            Rectangle screenRect = new Rectangle(x, y, width, height);
            BufferedImage image = robot.createScreenCapture(screenRect);
            String dirName = getParameter("capture.save.dirname");
            String home = System.getProperty("user.home");
            File dir = new File(home, dirName);
            dir.mkdirs();
            File path = new File(dir, dateFormat.format(new Date()) + "_" + name + ".png");
            putMessage(path.getAbsolutePath());

            ImageIO.write(image, "png", path);

        } catch (Exception e) {

            e.printStackTrace();
        }

        putMessage("<<<<<<< [END_execCapture]\t" + name);
    }

    public void printWindowInfomation() {

        Window[] windows = Window.getWindows();
        for (Window window : windows) {

            if (window instanceof Frame) {

                Frame frame = (Frame)window;
                System.out.println("Title" + frame.getTitle() + " width:" + frame.getWidth());
//                window.getToolkit().
            } else {

                System.out.println(window.getName());
            }
        }

        Container container = getParent();
        while (null != container) {

            if (container instanceof Frame) {

                Frame frame = (Frame)container;
                System.out.println(frame.getTitle());
//                window.getToolkit().
            }

            container = container.getParent();
        }
    }

    public void openSizeSpecifyWindow() {

        try {
            if (null == window) {

                window = new CaptureSizeWindow();
            }
            window.setVisible(true);
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
}
